
/**
 * finds the DOMWindow from the air native window
 * @param nativeWindow
 * @returns DOMWindow or null
 */
function domWindow(nativeWindow){
	return nativeWindow ? nativeWindow.stage.getChildAt(0).window : null;
}
/**
 * find the DOMWindow from startup.html.
 * this assumes that startup.html is the first nativeWindow in the application.
 * @returns DOMWindow
 */
function startupWindow(){
	return domWindow(air.NativeApplication.nativeApplication.openedWindows[0]);
} 

var settings = (function(){
    if(startupWindow().settings){
        return startupWindow().settings;
    }

    var xhr = $.ajax({
        url:'settings.json',
        async:false,
        dataType:'json'
    });
    //alert(xhr);
    return JSON.parse(xhr.responseText);
})();

if( settings.introspector ){
    document.write("<script type='text/javascript' src='lib/air/AIRIntrospector.js'><\/script>");
}
/*
    Following generates log(), info(), warn(), error() and dump() global functions
    which in the presents of air.Introspector go to it's Console other wise they go to
    window.runtime.trace().
 */
(function(){
    var slice = Array.prototype.slice,
    stringify = function(o){
        return typeof o === 'string' ? o : JSON.stringify(o);
    };
    ['log','info','warn','error','dump'].forEach(function(level){
        if( settings.debug ){
            window[level] = settings.introspector ? function(){
                var a = slice.call(arguments,0);
                if( air.Introspector ){
                    air.Introspector.Console[level].apply(air.Introspector.Console,a);
                }
            } : function(){
                var a = slice.call(arguments,0);
                window.runtime.trace.apply(window.runtime, a.map(stringify));
            }
        } else {
            window[level] = $.noop;
        }
    });
})();

// util namespace to express milli seconds consisely
// e.g. 5 minutes: ms.minutes(5)
//     30 seconds: ms.seconds(30)
// could in theory have been the more general interface of minutes(5).ms()& minutes(5).s()
// etc, not really needed at this point
var ms = {
    /**
     * Number of milliseconds in 'h' hours
     * @param h number of house
     * @returns number of milliseconds
     */
    hours:function(h){
        return ms.minutes(h * 60);
    },
	/**
	 * Number of milliseconds in 'm' minutes.
	 * @param m number of minutes
	 * @returns number of milliseconds
	 */
	minutes: function(m){
		return ms.seconds(m * 60);
	},
	/**
	 * Number of milliseconds in 's' seconds
	 * @param s number of seconds
	 * @returns number of milliseconds
	 */
	seconds: function(s){
		return s * 1000;
	}
};

$.ajaxSetup({timeout:ms.seconds(15)});

/*
 * Include the metoffice ns if not present instead of a direct reference to metoffice.data.js
 */
if( typeof window.metoffice === 'undefined'
	&& startupWindow().metoffice ){
	window.metoffice =  startupWindow().metoffice;
    window.override = startupWindow().override;
	log("Configured metoffice NS from parent window");
}

// monkey patch to add valuesFor(displayPeriod) to forecastDay
if( typeof metoffice.forecastDay.prototype.valuesFor === 'undefined'){
	metoffice.forecastDay.prototype.valuesFor = function(displayPeriod){
		return displayPeriod === metoffice.displayPeriod.DAY ? this.getDayValues() : this.getNightValues();
	};
}

var warningsRefreshTimeout = ms.minutes(settings.warningsRefreshMinutes);
var weatherRefreshTimeout = ms.minutes(settings.weatherRefreshMinutes);

function configureServer(){
	metoffice.setRootURL(settings.rootDataUrl);
}

function onClose() {
    var closing = new air.Event(air.Event.CLOSING, true, true);
    window.nativeWindow.dispatchEvent(closing);
    if (! closing.isDefaultPrevented()) {
        window.close();
	    air.NativeApplication.nativeApplication.icon.bitmaps = [];
		air.NativeApplication.nativeApplication.exit();
	}
}

function getELSParam(paramName) {
	try {
		var storedValue = air.EncryptedLocalStore.getItem(paramName);
		var result = storedValue.readUTFBytes(storedValue.bytesAvailable);
		
		return result;
	}
	catch (e) {
		return null;
	}
}

function setELSParam(paramName, value) {
	var bytes = new air.ByteArray(); 
	bytes.writeUTFBytes("" + value); 
	air.EncryptedLocalStore.setItem(paramName, bytes, false);	
}

/**
 * find the description of the weather.
 * @param weather numeric weather number
 * @returns localised weather description
 */
function weatherDescription(weather){
	return (weather === null ? labels.weather.unknown : labels.weather.description[weather]);
}

// here to be applied to the temp/wind/uv objects
//TODO move _appendTo to closure somewhere
var _appendTo = function($element){
	return $(this.toMarkup()).appendTo($element);
};

/**
 * Temperature object creator.
 * The result object has three methods in addition to the properties:
 *    toString()  - simple formatted string
 *    toMarkup()  - output wrapped in a span with spans wrapping the value and
 *    appendTo($) - append the markup to a jQuery object 
 * @param tempInC required temperature
 * @param requiredUnits either 'C' or 'F'
 * @param titlePrefix defaults to 'Temperature'
 * @param round defaults to true
 * @return Temperature object with properties 'value', 'units', 'displayUnits'
 */
var temperature = (function(){
	var Temperature = function(tempInC, requiredUnits, titlePrefix, round){
        if(tempInC!=null){
            if( typeof requiredUnits === 'undefined'){
                requiredUnits = moTemperatureUnits;
            }
            if( typeof titlePrefix === 'undefined') {
                titlePrefix = 'Temperature';
            }
            if( typeof round === 'undefined'){
                round = true;
            }
            var temp = tempInC;
            this.units = requiredUnits;
            this.displayUnits = labels.weather.temperature[requiredUnits];
            this.titlePrefix = titlePrefix;
            
            if (requiredUnits === "F") {
            log("tempInC",tempInC);
                temp = metoffice.utility.convert({quantity:temp,from:"c",to:"f", round:false});
            }
            if( round === true){
                temp = Math.round(temp);
            }
            temp = temp.toFixed(1);
            if( temp.slice(-1) == '0') {
                temp = temp.slice(0,-2);
            }
            this.value = temp;
        }else{
            this.value = null;
        }
	};
	Temperature.prototype = {
			toString : function(){
                var returnStr = "Unknown";
                if(this.value!=null){
                    returnStr = this.value + "\u00B0" + this.displayUnits;
                }
                return returnStr;
			},
			toMarkup : function(){
                var title = "Unknown";                
                if(this.value!=null){
                    title = this.titlePrefix + ' ' + this.value+ "\u00B0" + this.displayUnits;
                }
                var returnStr = "<span class='temperature' title='"+ title +"'><span class='value'>";
                if(this.value!=null){
                    returnStr += this.value+ "</span>\u00B0<span class='units'>" + this.displayUnits;
                }else{
                    returnStr += "-";
                }
                returnStr += "</span></span>";
				return returnStr;
			},
			appendTo : _appendTo
	};
	return function( tempInC, requiredUnits, titlePrefix, round ){
		return new Temperature(tempInC, requiredUnits, titlePrefix, round);
	};
})();

/**
 * Wind object creator.
 * The result object has three methods in addition to the properties:
 *    toString()  - simple formatted string
 *    toMarkup()  - output wrapped in a span with spans wrapping the value with img and label tags
 *    appendTo($) - append the markup to a jQuery object 
 * @param speed required speed in mph
 * @param direction string in the form compass point form eg 'N' or 'NNE'
 * @param requiredUnits either M - mph, K - kph, N - knots
 * @return Wind object with properties 'value','speed','direction', 'units','displayUnits'
 * (note 'value' and 'speed' are the same value)
 */
var wind = (function(){
	var Wind = function(speed, direction, requiredUnits, displayText){
        if(speed!=null){
            if( typeof requiredUnits === 'undefined'){
                requiredUnits = moSpeedUnits;
            }
            if (requiredUnits === "K") {
                speed = metoffice.utility.convert({quantity:speed,from:"mph",to:"kmh"});
            } else if (requiredUnits === "N") {
                speed =  metoffice.utility.convert({quantity:speed,from:"mph",to:"kts"});
            }		
            this.value = this.speed = Math.round(speed);
            this.units = requiredUnits;
            this.displayUnits = requiredUnits;
            this.direction = (direction == "missing" || direction == null) ? "VRB" : direction;
            this.displayText = displayText || requiredUnits ;
        }else{
            this.value = null;
            this.direction = "VRB";
        }
	};
	Wind.prototype = {
			toString : function(){
                if(this.value!=null){
                    return internationalDirection(this.direction) + " " + this.value + " " + this.displayText;
                }else{
                    return "Unknown";
                }
			},
			toMarkup : function(){
                var markup = "<div class='wind' title='" + this.toString() + "'>" +
						"<img src='/lib/images/windSymbols/" + this.direction + ".png'/>" +
						// TODO review use of label for wind overlay
						"<label class='qty'>";
                    if(this.value != null){
                        markup += this.value;
                    }else{
                        markup += "-";
                    }
                    markup += "</label></div>";
                return markup;
			},
			appendTo : _appendTo
	};
	return function(speed, direction, requiredUnits, displayText){
		return new Wind(speed, direction, requiredUnits, displayText);
	};
})();

/**
 * UV object creator.
 * The result object has three methods in addition to the properties:
 *    toString()  - simple formatted string
 *    toMarkup()  - output wrapped in a span with spans wrapping the value with img and label tags
 *    appendTo($) - append the markup to a jQuery object 
 * @param speed required speed in mph
 * @param direction string in the form compass point form eg 'N' or 'NNE'
 * @param requiredUnits either M - mph, K - kph, N - knots
 * @return Wind object with properties 'value','speed','direction', 'units','displayUnits'
 * (note 'value' and 'speed' are the same value)
 */
var uv = (function(){
	var uvSymbol =     ["", "green", "green", "yellow", "yellow", "yellow", "orange", "orange", "red", "red", "red", "purple"]; // 0 left blank as UV values range from 1 to 11;
	var uvTextColour = ["", "white", "white", "#555555", "#555555", "#555555", "black", "black", "white", "white", "white", "white"]; // 0 left blank as UV values range from 1 to 11;
	
	var UV = function(uv){
		this.value = parseInt(uv,10);
	};
	UV.prototype = {
		toString : function(){
			return this.value > 0 ? labels.weather.uv[""+this.value] : labels.weather.uv["unknown"];
		},
		toMarkup : function(){
			return "<div class='uv' title='" + this.toString() + "'>" + 
					(this.value > 0 ? 
							("<img src='/lib/images/uvSymbols/" + uvSymbol[this.value] + ".png'/>" +
							 "<label style='color:" + uvTextColour[this.value] + "' class='qty'>" + this.value + "</label>")  
							 : "-") + 
					 "</div>";
		},
		appendTo : _appendTo 
	};
	
	return function(uv){
		return new UV(uv);
	};
})();

/**
 * Weather object creator.
 * The result object has four methods in addition to the property 'value':
 *    toString()   - simple formatted string
 *    toMarkup()   - output wrapped in a span with spans wrapping the value with img and label tags
 *    getIconUrl() - application relative url for the weather icon
 *    appendTo($)  - append the markup to a jQuery object 
 * @param speed required speed in mph
 * @param direction string in the form compass point form eg 'N' or 'NNE'
 * @param requiredUnits either M - mph, K - kph, N - knots
 * @return Wind object with properties 'value','speed','direction', 'units','displayUnits'
 * (note 'value' and 'speed' are the same value)
 */
var weather = (function(){
	var Weather = function(weather){
		this.value = weather;
	};
	Weather.prototype = {
			toString:function(){
				return weatherDescription(this.value);
			},
			getIconUrl:function(){
				return "/lib/images/wxsymbols/w" + ((this.value!==null)?this.value:"missing") + ".png";
			},
			toMarkup:function(){
				return "<span class='weather' title='" + weatherDescription(this.value) + "'>" + 
						"<img src='" + this.getIconUrl() + "'/>"  + 
					"</span>";
			},
			appendTo : _appendTo
	};
	return function(weather){
		return new Weather(weather);
	};
})();

function setupLanguage(language, callback) {
    if( startupWindow().labels){
        setupLabels(startupWindow().labels,callback);
        return ;
    }
	$.ajax({
		url: "/lib/data/language/" + language + ".txt",
		method: "GET",
		dataType: "json",
		success: function(data) {
			setupLabels(data, callback);
		},
		timeout: 15000
	});
}

function setupLabels(data, callback) {
    window.labels = data;
	callback(data);
}

function internationalDay(requiredDate){
	return labels.calendar.dayIndex[requiredDate.getDay()];
}

function internationalDate(requiredDate) {
    var date = requiredDate.getDate();
    var month = requiredDate.getMonth();
    var year = requiredDate.getFullYear();

    return internationalDay(requiredDate) + " " + date + " " + labels.calendar.monthIndex[month] + " " + year;
}

function internationalDirection(bearing) {
    var output = "";
    for (var i = 0; i < bearing.length; i++) {
        output += labels.direction[bearing.charAt(i)];
    }
    return output;
}
function internationalDisplayPeriod(displayPeriod) {
	return displayPeriod == metoffice.displayPeriod.DAY ? 
				labels.calendar.today: labels.calendar.tonight;
}

function getDefaultParams() {
	// get parameters
	getParam("moLanguage", "en");
	getParam("moMapDisplay", "true");
	getParam("moMapsRequired", "Precip Rate LR,Pressure MSL,Temperature LR");
	getParam("moListStyle", "vertical"); //horizontal == compact, vertical == normal verbose mode
	getParam("moDays", 5);
	getParam("moFSSI", 352409);
	getParam("moTemperatureUnits", "C");
	getParam("moSpeedUnits", "M");
	getParam("moShowWind", "true");
	getParam("moShowUV", "true");
    getParam("moShowFeelsLike", "true");
    getParam("moAllowUserLocation", "true");
}

function getParam(paramName, defaultValue) {
	var actualValue = getELSParam(paramName);
	if (actualValue == null) {
		setELSParam(paramName, defaultValue);
		actualValue = defaultValue;			
	}
	window[paramName] = actualValue;
}


// following is taken from http://sevenwire.com/blog/2009/08/26/adobe-air-opening-external-links-in-another-browser.html
function openExternalURL(href) {
    var request = new air.URLRequest(href);
    try {            
        air.navigateToURL(request);
    } catch (e) {
        error("openExternalURL",href, e);
        throw e;
    }
}


$(function(){
	// install a global listerner to open links in an external browser
    $('a[href^=http]').live("click", function(event){
        event.preventDefault();
        openExternalURL(this.href);
    });
    function adjust(adjustment, name){
    	return function(){$(this)[adjustment + "Class"](name);};
    }
	// this just sorts out the rollovers for the header
	$('#header .ui-icon').hover(adjust('add', 'hover'),adjust('remove','hover'));
//		.bind('mousedown',adjust('add', 'active'))
//		.bind('mouseup', adjust('remove', 'active'));
    if( air.NativeWindow.supportsTransparency ){
        $(document.body).addClass("supports-transparency");
    }
});

function timeFormat(date){
	// taken from the obs output
	// TODO review time format
	return ((date.getHours()<10)?"0":"")+date.getHours()+((date.getMinutes()<10)?"0":"")+date.getMinutes();
}

function visibilityToWords(vis){
	return labels.weather.visibility[metoffice.utility.convertVisibiltyToWords(vis)];
}

function internationalPlace(place){
   if(typeof labels.places != "undefined" 
    	&& typeof labels.places[place] != "undefined") {
        return labels.places[place];
    }
   return place;
}

// common window handling 

function defaultNativeWindowOptions(){
	var options = new air.NativeWindowInitOptions(); 
	options.transparent = true;
	options.systemChrome = "none";
	options.type = "lightweight";
	options.resizable = false;
	options.maximizable = false;
	return options;
}

function makeWindowLoaderModal(htmlLoader) {
    function triggerModalOptions(e) {
        var w = e.target.window;
        w.nativeWindow.activate();
        w.nativeWindow.alwaysInFront = true;
        w.nativeWindow.addEventListener(
            'deactivate',
            function (e) {w.nativeWindow.activate();}
        );
        w.nativeWindow.addEventListener(
            'displayStateChanging',
            function (e) {e.preventDefault();}
        );
        w.nativeWindow.addEventListener(
            'closing',
            function (e) {e.preventDefault();}
        );

        e.target.removeEventListener(air.Event.COMPLETE, triggerModalOptions);
    };
    htmlLoader.addEventListener(air.Event.COMPLETE, triggerModalOptions);
}

/**
 * Window position creator function.
 * Can be passed either x,y,w,h or an object with 'x','y','width' & 'height'
 * properties.
 *
 * Properties:
 *  x
 *  y
 *  width
 *  height
 *
 * Methods:
 *  rectangle([overrides]) - convert to a air.Rectangle with optional overrides
 *  merge(other) - return a new position with values from 'other' overriding the values of this.
 *                 note must use return value, the original object is left unchanged
 */
var position = (function(){
    var Position = function(x,y,width,height){
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    };
    function choose(a,b){
        return typeof a === 'number' ? a : b;
    }
    Position.prototype = {
        rectangle:function(overrides){
            overrides = overrides || {};
            return new air.Rectangle(choose(overrides.x, this.x),
                                        choose(overrides.y, this.y),
                                        choose(overrides.width, this.width),
                                        choose(overrides.height, this.height));
        },
        merge:function(other){
            return position(choose(other.x, this.x),
                            choose(other.y, this.y),
                            choose(other.width, this.width),
                            choose(other.height, this.height));
        },
        centerOver:function(relative){
            return this.merge({
                x:(relative.width / 2) - this.width/2 + relative.x,
                y:(relative.height / 2) - this.height/2 + relative.y
            });
        },
        ensureVisibleOnScreen:function(options){
            options = options || {};
            var rectangle = this.rectangle(options),
                screens = air.Screen.screens,
                bounds,
                onScreen,
                fullyOnScreen = !!options.fullyOnScreen,
                min = Math.min,
                max = Math.max;

            onScreen = screens.some(fullyOnScreen ? function(screen){
                return screen.visibleBounds.containsRect(rectangle);
            } : function(screen){
                return screen.visibleBounds.intersects(rectangle);
            });

            if( onScreen ){
                return this;
            }
            // off screen position on mainScreen
            bounds = air.Screen.mainScreen.visibleBounds;

            return this.merge({
                x:min(bounds.width-this.width, max(0,this.x)),
                y:min(bounds.height-this.height, max(0,this.y))
            });
        }
    };
    return function(x,y,width,height){
        var p ;
        if( arguments.length == 4 ){
            return new Position(x,y,width,height);
        } else {
            // assume first parameter is a position like object
            p = x;
            return new Position(p.x,p.y,p.width,p.height);
        }
    };
})();

// static method on position
position.screenCenter = function(width,height){
    var screenBounds = air.Screen.mainScreen.visibleBounds;

    return position({
        x:(screenBounds.width / 2) - width/2,
        y:(screenBounds.height / 2) - height/2,
        width:width,
        height:height
    });
};

var mainWindow = startupWindow().mainWindow || {
    instance: null,
    opening: false,
    position:function(){
        var pos;
        if( this.instance ){
            pos = position(this.instance.bounds);
        }else{
            pos = localOptions.object("mainWindow", position.screenCenter(470,360), position);
        }

        return pos.ensureVisibleOnScreen({height:30});
    },
    show:function(options){
        var self = this;
        this.opening = true;
        try{
            options = options || {};
            log("mainWindow.show()", options);

            if( this.bringToFront(options)){
                return ;
            }

            var p = this.position();
            log(JSON.stringify(p));

            var newHTMLLoader = air.HTMLLoader.createRootWindow(true, defaultNativeWindowOptions(), true, p.rectangle());
            if( options.showWarnings ){
                newHTMLLoader.addEventListener("complete", function(event){
                    event.target.window.showWarnings({syncButtons:true});
                });
            }
            newHTMLLoader.load(new air.URLRequest("mainWindow.html"));
            this.instance = newHTMLLoader.window.nativeWindow;

            this.instance.addEventListener("move", moveEventHandler("mainWindow"));

            this.instance.addEventListener(
                air.Event.CLOSE, function(){
                    self.instance = null;
                    if( air.NativeApplication.supportsDockIcon){
                        // if we have a dock icon, ie (on a mac) then quit the application on closing the main window
                        nativeApplication.exit(0);
                    }
                }
            );
        }finally{
            this.opening = false;
        }
    },
    bringToFront:function (options){
        options = options || {};
        if (this.instance) {
            // move to display position
            this.instance.bounds = this.position().rectangle();

            // feels dirty but this is effective as a way to poke the window
            // to the front

            this.instance.alwaysInFront = true;
            this.instance.alwaysInFront = false;
            log("bringToFront()", this.instance);
            if( options.showWarnings ){
                domWindow(this.instance).showWarnings({syncButtons:true});
            }
            return true;
        }
        return false;
    },
    isVisible:function(){
        return this.opening || !!this.instance;
    }
};

var newsWindow = startupWindow().newsWindow || {
    instance:null,
    show:function(){
        var self = this;
        if(this.instance == null){

            var p = localOptions.object("news", mainWindow.position()
                                                            .merge({width:440,height:400})
                                                            .centerOver(mainWindow.position()), position);

            var newHTMLLoader = air.HTMLLoader.createRootWindow(true, defaultNativeWindowOptions(), true,
                    p.ensureVisibleOnScreen().rectangle());

            newHTMLLoader.load(new air.URLRequest("news.html"));

            this.instance = newHTMLLoader.window.nativeWindow;

            this.instance.addEventListener("move", moveEventHandler("news"));

            this.instance.addEventListener(
                air.Event.CLOSE, function(){self.instance = null;}
            );
        } else {
            this.instance.activate();
        }
    }
};

var modalWindow = (function(){
    var visibleWindow = null;

    var ModalWindow = function(html, point){
    	this.instance = null;
    	this.html = html;
    	this.point = point; 
    };
    
    ModalWindow.prototype = {
		show : function(){
	        var self = this;

	        if( this.instance == null){
		        if(visibleWindow !== null){
		        	visibleWindow.hide();
		        }
	            var p = mainWindow.position()
                                    .merge(this.point)
                                    .centerOver(mainWindow.position())
                                    .ensureVisibleOnScreen({fullyOnScreen:true});

	            var newHTMLLoader = air.HTMLLoader.createRootWindow(true, defaultNativeWindowOptions(), true,
	                    p.rectangle());
	            newHTMLLoader.load(new air.URLRequest(this.html));

	            this.instance = newHTMLLoader.window.nativeWindow;

                makeWindowLoaderModal(newHTMLLoader);
	        }else{
	            this.instance.activate();
	        }
	        visibleWindow = this;
	    },
	    hide : function(){
	        if( this.instance ){
                networkMonitor.clear(this.html);
	            this.instance.close();
	            this.instance = null;
	            visibleWindow = null;
	        }
	    }  
    };
    return function( html, width, height ){
        return new ModalWindow(html, width, height);
    };
})();

/** 
 * options window.
 * Not quite the same as the newsWindow as the options window
 * is modal the is no point maintaining a common reference across
 * startup and mainWindow.
 */
var optionsWindow = startupWindow().optionsWindow || modalWindow("options.html", {width:350,height:420});

var aboutWindow = startupWindow().aboutWindow || modalWindow("about.html", {width:320,height:320});

var updateWindow = startupWindow().updateWindow || modalWindow("update.html", {width:400,height:250});

/**
 * Supplies defaults for the jQuery tooltip() function.
 *    "topright" to be used for items in teh topright of the view port
 * @param name optional (use null for default) or just omit
 * @param options overrides for the options object
 * @returns options
 */
function defaultTooltip(name, options){
	var opts = {effect:'toggle'},
		k;
	
	if( name === "topright"){
		opts = {
            position: "bottom left",
            offset:[10,20],
            effect:'toggle', 
            tipClass:"topright-tooltip"
        };
	}
    if( name === "topleft"){
        opts = {
            position:"bottom right",
            offset:[10,-20],
            effect:"toggle",
            tipClass:"topleft-tooltip"
        };
    }
    if( name === "left"){
        opts = {
            position:"center right",
            offset:[-10,0],
            effect:"toggle",
            tipClass:"left-tooltip"
        };
    }

	if( options ){
		for( k in options ){
			if( options.hasOwnProperty(k)){
				opts[k] = options[k];
			}
		}
	}
	return opts;
}

var localOptions = (function(){
	var els = air.EncryptedLocalStore;
	return {
		get:function(name,type){
			var byteArray = els.getItem(name);
			if( byteArray ){
				switch(type){
                case 'date':
                    return new Date(byteArray.readDouble());
				case 'string':
					return byteArray.readUTF();
				case 'number':
					return byteArray.readDouble();
                case 'boolean':
                    return byteArray.readBoolean();
				case 'object':
					return byteArray.readObject();
				}
			}
			return byteArray;
		},
		set:function(name, value){
			var byteArray;
			if( typeof value.bytesAvailable === 'number' ){
				// assume it is already a ByteArray
				byteArray = value;
			} else {
				byteArray = new air.ByteArray();
                if( value instanceof Date){
                    value = value.getTime();
                }
				switch(typeof value){
					case 'string':
						byteArray.writeUTF(value);
						break;
					case 'number':
						// you did know that all numbers are doubles in JS
						byteArray.writeDouble(value);
						break;
                    case 'boolean':
                        byteArray.writeBoolean(value);
                        break;
					default:
						byteArray.writeObject(value);
						break;
				}
			}
			els.setItem(name, byteArray);
		},
        remove:function(name){
            els.removeItem(name);
        },
		string:function(name, defaultValue){
			defaultValue = typeof defaultValue === 'string' ? defaultValue : null;
			return this.get(name, 'string') || defaultValue;
		},
		number:function(name, defaultValue){			
			defaultValue = typeof defaultValue === 'number' ? defaultValue : null;
			return this.get(name, 'number') || defaultValue;
		},
		object:function(name, defaultValue, fn){
			defaultValue = typeof defaultValue === 'object' ? defaultValue : null;
			var val = this.get(name, 'object') || defaultValue;
            if( typeof fn == 'function' && val){
                return fn(val);
            }
            return val;
		},
        date:function(name, defaultValue){
            defaultValue = defaultValue instanceof Date ? defautlValue : null;
            return this.get(name, 'date') || defaultValue;
        },
        boolean:function(name,defaultValue){
            defaultValue = typeof defaultValue === 'boolean' ? defaultValue : false;
            var value = this.get(name,'boolean');
            return value === null ? defaultValue : value;
        }
	};
})();

/**
 * Holder for the current location object.
 * use: get() and set() methods to access.
 * will default to London if no previously saved location is present
 */
var currentLocation = startupWindow().currentLocation || {
		_location:null,
        _nearestObservation:null,
		get:function(){
			if( this._location === null){
				var values = localOptions.object("moCurrentLocation",{
					/* default to london */
					"id":352409,
					"altitude":11,
					"point":{"lat":51.5081,"lon":-0.1248},
					"name":"London",
					"region":metoffice.utility.getRegionFromId("se").values(),
					"forecastZoomLevel":5,
					"showOnMap":1,
					"type":"Residential",
                    observationZoomLevel:null
				});
//                values.point = new metoffice.point(values.point._values);
//				values.region = metoffice.utility.getRegionFromId(values.region);
				this._location = new metoffice.location(values);
			}
			return this._location;
		},
		set:function(locationObj){
            if( locationObj ){
                if( this._location === null
                        || this._location.getId() !== locationObj.getId()){
                    log("currentLocation removing observation");
                    this._nearestObservation = null;
                    localOptions.remove("moCurrentObservation");
                }
                localOptions.set("moCurrentLocation", locationObj._values);
                this._location = locationObj;
            }
		},
        nearestObservation:function(options){
            var self = this,
                defaultValues = null,
                values;
            if( this._nearestObservation === null) {
                // attempt to get the object from the localOptions, being care to only use the default of northholt
                // if the currentLocation is London

                if( this.get().getId() === 352409){
                    defaultValues = {
                        "dist":20.559809107261326,
                        "location":{
                            "id":3672,
                            "altitude":33,
                            "point":{ "lat":51.548, "lon":-0.415 },
                            "name":"Northolt",
                            "region":metoffice.utility.getRegionFromId("se").values(),
                            "forecastZoomLevel":9,
                            "type":"Observing Site",
                            "observationZoomLevel":8
                        }
                    };
                }
                values = localOptions.object("moCurrentObservation", defaultValues);
                if( values === null ){
                    log("nearestObservation loading nearestObservationLocation via api");
                    this.get().getNearestObservationLocation({
                        success:function(result){
                            self._nearestObservation = result;
                            
                            if(result.location !== null && result.location._values.region!==null){
                            	result.location._values.region = result.location.getRegion().getId();
                            }

                            localOptions.set("moCurrentObservation", {
                                dist:result.dist,
                                location:result.location.values()
                            });
                            options.success(result);
                        },
                        error:options.error
                    });
                    return ; // exit with out directly calling the success method
                } else {
                    values.location = new metoffice.location(values.location);
                    this._nearestObservation = values;
                }
            }
            options.success(this._nearestObservation);
        },
        reset: function(){
    		this._location = null;
            this._nearestObservation = null;
        }
};

var networkMonitor = startupWindow().networkMonitor || (function(){
    var urlMonitor = null;
    var upListeners = {};
    var downListeners = {};
    function register(listeners, name, callback){
        listeners[name] = listeners[name] || [];
        listeners[name].push(callback);
    }
    function invoke(fn){
        try{
            fn();
        }catch(e){
            info("failed to invoke networkMonitor listener",e);
        }
    }
    function trigger(listeners){
        var k;
        for( k in listeners){
            if( listeners.hasOwnProperty(k)){
                listeners[k].forEach(invoke);
            }
        }
    }

    if( typeof air.URLMonitor === 'function' ){
        urlMonitor = new air.URLMonitor(new air.URLRequest(settings.updateInfoUrl),[200,302,301,404,405,403]);
        urlMonitor.addEventListener("status", function(){
            log("urlMonitor status changed :" + urlMonitor.available, urlMonitor);
            trigger(urlMonitor.available ? upListeners : downListeners);
        });
        urlMonitor.start();
    }else{
        urlMonitor = {available:true};
    }
    return {
        up:function(name,callback,suppressImmediate){
            register(upListeners, name, callback);
            if( !!!suppressImmediate && urlMonitor.available){
                invoke(callback);
            }
        },
        down:function(name,callback, suppressImmediate){
            register(downListeners, name, callback);
            if( !!!suppressImmediate && !urlMonitor.available){
                invoke(callback);
            }
        },
        clear:function(name){
            log("networkMonitor.clear() : " + name);
            delete upListeners[name];
            delete downListeners[name];
        },
        available:function(){
            return urlMonitor.available;
        }
    };
})();


var refreshTimers = (function(){
    var timers = [];
    // array of pairs of timer and fn.. allows removal of the timers by fn
    var timersAndFn = [];
    return {
        /**
         * Create a new timer to call fn every 'refreshInterval' ms.
         * @param refreshInterval ms seconds between calling the function
         * @param fn function to call
         * @returns the started timer
         */
        startTimer: function (refreshInterval, fn){
            var timer = new air.Timer(refreshInterval, 0);
            timer.addEventListener("timer", fn);
            timer.start();
            timers.push(timer);
            timersAndFn.push({
                timer:timer,
                fn:fn
            });
            return timer;
        },
        removeTimer:function(fn){
            var i,len = timersAndFn.length,
                    pair;
            for( i = 0; i < len ; i++ ){
                pair = timersAndFn[i];
                if( pair.fn === fn){
                    pair.timer.stop();
                    timersAndFn.splice(i,1);
                    timers.splice(timers.indexOf(pair.timer),1);
                    log("removed timer");
                    break;
                }
            }
        },
        stop:function(){
            timers.forEach(function(timer){timer.stop();});
        },
        restart:function(){
            timers.forEach(function(timer){
                timer.reset();
                timer.start();
            });
        },
        count:function(){return timers.length;}
    }
})();

function moveEventHandler(name){
    return function(e){
        var b = e.afterBounds;
        localOptions.set(name, {
            x:b.x,
            y:b.y,
            width:b.width,
            height:b.height
        });
    }
}

var enableButton = function(id, enable){
	if (enable) {
		$(id+":button").removeAttr("disabled"); 
	}else{
		$(id+":button").attr("disabled", "disabled"); 
	}		
};

var updater = startupWindow().updater || {
//    latestVersionCheckUrl :"http://localhost/air/metoffice/versioning.xml",
    latestVersionCheckUrl :settings.updateInfoUrl,
    applicationName : "DesktopWidget.air",
    applicationVersion : null,
    latestUpdate : null,
    updaterUrl : settings.updateFileUrl,
    stream : null,
    updateFile : null,
    versionLength : 3,
    updateAvailable:function(){
        var latest = this.latestUpdate;
        return latest ? latest.version.compareTo(this.getApplicationVersion()) > 0 : false;
    },
    updateErrorHandler : function(){},
    version : (function(){
        var Version = function(textVersion){
            this.versionText = textVersion;
            this.versionParts = textVersion.split(".").map(function(v){return parseInt(v,10);});
        };

        Version.prototype = {
            compareTo : function(toVersion){
                var i,
                    parts = this.versionParts,
                    otherParts = toVersion.versionParts;

                for( i = 0 ; i < parts.length; i++){
                    if( i == otherParts.length ){
                        // we are greater
                        return 1;
                    }
                    if(parts[i] < otherParts[i]){
                        // other is greater
                        return -1;
                    }
                    else if(parts[i] > otherParts[i]){
                        // we are greater
                        return 1;
                    }
                }
                if( i < otherParts.length ){
                    // other has more parts
                    return -1
                }
                return 0;
            }

        };
        return function( textVersion ){
            return new Version(textVersion);
        };
    })(),
    /**
     * Lazy loads and returns the current application version
     * @returns A Version object.
     * @throws error if fails to find the application version
     */
    getApplicationVersion : function () {
        // This will get the version of the currently installed application
        if(this.applicationVersion === null)
        {
            var appXML = air.NativeApplication.nativeApplication.applicationDescriptor;
            var xmlObject = new DOMParser().parseFromString(appXML, "text/xml");
            this.applicationVersion = this.version(xmlObject.getElementsByTagName('version')[0].firstChild.nodeValue);
        }

        return this.applicationVersion;
    },
    /**
     * Returns an object literal containing the latest update information.
     * @returns {version:Version, note:string, fileURL:url}
     */
    fetchLatestUpdate: function (onSuccess, onError) {
        var self = this;
        $.ajax({
            type: "GET",
            url: self.latestVersionCheckUrl,
            dataType: "xml",
            error: function(XMLHttpRequest, textStatus, errorThrown){
                if( onError ){
                    onError("Unable to retrieve the latest version info");
                }
            },
            success: function(data) {
                var releaseNote = null;
                var versionText = null;
                var url = null;
                versionText = $(data).find('version').text();
                url = $(data).find('url').text();
                releaseNote = $(data).find('description').text();

                if(typeof versionText != "undefined" && versionText!=null && versionText.length > 0){
                    self.latestUpdate = {
                        version:self.version(versionText),
                        url:url,
                        note:releaseNote
                    };
                    onSuccess(self.latestUpdate);
                }
                else{
                    if(onError){
                        onError("Unable to retrieve the latest version info");
                    }
                }
            }
        });
    },

    checkForUpdate : function (onSuccess, onError) {
        var currentVersion = this.getApplicationVersion();
        this.fetchLatestUpdate(
            function(versionInfo){
                onSuccess((currentVersion.compareTo(versionInfo.version) < 0)?versionInfo:null);
            },
            onError
        );
    },

    /**
     * @param options{
     * 	updatingStatus : function(event)
     * 	onError : function(error)
     * }
     */
    startDownload : function (options) {
        var self = this;
        if(typeof options.onError == "function"){
            this.updateErrorHandler = options.onError;
        }else{
            this.updateErrorHandler = function(error){};
        }
        this.fetchLatestUpdate(
            function(versionInfo){
                if(versionInfo ){
                    try{
                        self.stream = new air.URLStream();

                        /*
                            This event is recommend to give the end user continuous
                            feedback about how the update goes
                        */
                        if(typeof options.updatingStatus == "function"){
                            self.stream.addEventListener(air.ProgressEvent.PROGRESS, options.updatingStatus);
                        }
                        if( typeof options.complete == 'function'){
                            self.stream.addEventListener(air.Event.COMPLETE, options.complete);
                        }
                        self.stream.addEventListener("ioError", function(e){
                            error(e);
                            self.updateErrorHandler("Failed to download the update");
                        });

                        self.stream.load( new air.URLRequest(self.updaterUrl + versionInfo.version.versionText + ".air") );
                    }
                    catch(error){
                        self.updateErrorHandler("Failed to download update - "+error);
                    }
                }else{
                    self.updateErrorHandler("Failed to download update");
                }
            },
            self.updateErrorHandler
        );
    },

    installUpdate : function () {
        var self = this;
        try{
            var ba = new air.ByteArray(),
                stream = this.stream,
                updateFile = air.File.applicationStorageDirectory.resolvePath(this.applicationName),
                fileStream = new air.FileStream();

            stream.readBytes(ba, 0, stream.bytesAvailable);

            fileStream.addEventListener( air.Event.CLOSE, function(){
                var airUpdater = new air.Updater();
                // Notice that the version name has to be present as a second parameter
                try{
                    airUpdater.update(updateFile, self.latestUpdate.version.versionText);
                }catch(error){
                    self.updateErrorHandler("An error occurred whilst launching the installer "+error);
                }
            } );
            fileStream.openAsync(updateFile, air.FileMode.WRITE);
            fileStream.writeBytes(ba, 0, ba.length);
            fileStream.close();
        }
        catch(error){
            this.updateErrorHandler("Failed to download update - "+ error);
        }
    },

    cancelDownload:function(){
        if( this.stream){
            this.stream.close();
            this.stream = null;
        }
    }
};


/**
 * Return input as a string with the first char uppercased and the rest lowest.
 *
 * THING -> Thing
 * thing -> Thing
 * null  -> ""
 * ""    -> ""
 *
 * @param v value to capitalise
 * @return string not null
 */
function capitalise(v){
    if( !v ){
        return "";
    }
    if( typeof v !== 'string') {
        v = v.toString();
    }
    v = v.toLowerCase();

    return v.slice(0,1).toUpperCase() + v.slice(1);
}

function initialiseWindow(options){
    $("#header").bind('mousedown', function() { window.nativeWindow.startMove();});

    if( options.close ){
        $("#headerclose,#aboutClose,#cancel").click(options.close);
    }
    $('#header a, #header .windowcontrol').bind('mousedown', function(e){e.stopPropagation();});

    $('img').live('contextmenu',function(e){e.preventDefault();});
}