
Function.prototype.inheritsFrom = function( parentClassOrObject )
{
  if ( parentClassOrObject.constructor == Function )
  {       // Normal Inheritance
    this.prototype = new parentClassOrObject;
    this.prototype.constructor = this;
    this.parentConstructor = parentClassOrObject;
    this.prototype.parent = parentClassOrObject.prototype;
  }
    else   {
      // Pure Virtual Inheritance
      this.prototype = parentClassOrObject;
      this.prototype.constructor = this;
      this.parentConstructor = parentClassOrObject;
      this.prototype.parent = parentClassOrObject;
    }
  return this;
};

override = function(original, overrides) {
	var newObj;

	if((typeof original)=='function')
	{
		newObj = new function(){};
	}
	else{
		newObj = {};
	}

	for ( var m in original)
	{
		newObj[m] = original[m];
	}
	if (overrides) {
		for ( var method in overrides) {
			newObj[method] = overrides[method];
		}
	}

	return newObj;
};

//This is a base class for enum types.
EnumBase = function(val, display) {
	this.constructor.isEmpty = function(v, allowBlank) {
	    if (typeof v === 'object' || typeof v === 'function') {
	        var i;
	        for (i in v) {
	            if (v.hasOwnProperty(i)) {
	                return false;
	            }
	        }
	        return true;
	    } else {
	        // not an object or function - same as before
	        return v === null || v === undefined || (! allowBlank ? v === '' : false);
	    }
	};

	this.constructor.getKeyByValue = function(c, val) {
	    for (key in c) {
	        var obj = c[key];
	        if (obj instanceof c && obj.getValue() === val) {
	            return (key);
	        }
	    }
	    return (null);
	};

	if (EnumBase.isEmpty(this.constructor.items)) {
        this.constructor.items = [];
    }
    this.constructor.items.push(this);

    var _val = val;

    function _getValue()
    {
    	return _val;
    }

    this.getValue = _getValue;

    this.toJSON = function() {
        return (val);
    },

    this.toString = function(throwIfNotFound) {
        if (! EnumBase.isEmpty(display)) {
            return (display);
        } else {
            var key = EnumBase.getKeyByValue(this.constructor, this.getValue());
            if (! EnumBase.isEmpty(key))
                return (key);
            else if (throwIfNotFound)
                throw val + ' is not a valid value.';
            else
                return (this.val);
        }
    };

    this.constructor.parse = function(str) {
    	if(str !== null)
    	{
	        var val = this[str];
	        if (val != undefined) {
	            return (val);
	        } else {
	            var key = EnumBase.getKeyByValue(this, str);
	            if (! EnumBase.isEmpty(key))
	                return (this[key]);
	        }
    	}
    	return null;
	};
};

// This is the main metoffice namespace, which contains all the library classes and static methods
var metoffice = (function() {

	//CONSTANTS
	var _fqRoot = "http://www.metoffice.gov.uk";
	var _urlRoot = "";
	var _urlSuffix = null;
	var defaultTimeout = 15000;

	var _daylightSavingURL;
	var _displayPeriodURL;
	var _extremeURL;
	var _forecastURL;
	var _forecasterCommentaryURL;
	var _locationURL;
	var _locationDetailURL;
	var _mountainForecastURL;
	var _nationalParkForecastRegionURL;
	var _observationURL;
	var _regionalForecastURL;
	var _warningsURL;

	_setURLs();

	var _extremeCacheKey = "extreme";
	var _forecastCacheKey = "forecast";
	var _forecasterCommentaryCacheKey = "forecasterCommentary";
	var _locationCacheKey = "location";
	var _locationDetailCacheKey = "locationDetail";
	var _mountainForecastCacheKey = "mountainForecast";
	var _nationalParkCacheKey = "nationalPark";
	var _nationalParkForecastCacheKey = "nationalParkForecast";
	var _observationCacheKey = "observation";
	var _timezoneCacheKey = "timezone";
	var _regionalForecastCacheKey = "regionalForecast";
	var _warningsCacheKey = "warning";

	var _defaultSpeedUnit = "mph";
	var _defaultTempUnit = "c";
	var _defaultPressureUnit = "hpa";
	var _defaultDistanceUnit = "m";
	var _defaultHumidityUnit = "%";
	var _defaultRainUnit = "mm";
	var _defaultHoursUnit = "hours";

	//PRIVATE MEMBER VARIABLES
	var _cache = {};	// Private member of self exec. function, to hold private JS cache.
	var _allowDataCaching = true;
	var library = {};	// API built in this structure, then returned, to become the public API.

	var warningRetrievalTime = null;

	//ENUMS
	var displayPeriod = function(val, display){
		displayPeriod.parentConstructor.call(this, val, display);
	};
	displayPeriod.inheritsFrom( EnumBase );

	displayPeriod.DAY = new displayPeriod(0, 'Day');
	displayPeriod.NIGHT = new displayPeriod(1, 'Night');

	//Extreme Types
	var extremeType = function(val, display, defaultUom){
		extremeType.parentConstructor.call(this, val, display);
		var _defaultUom=defaultUom;
		function _getDefaultUom()
		{
		  return _defaultUom;
		}
		this.getDefaultUom = _getDefaultUom;
	};
	extremeType.inheritsFrom( EnumBase );
	extremeType.HMAXT = new extremeType(0, 'Highest max temp', _defaultTempUnit);
	extremeType.LMAXT = new extremeType(1, 'Lowest max temp',_defaultTempUnit);
	extremeType.LMINT = new extremeType(2, 'Lowest min temp',_defaultTempUnit);
	extremeType.HRAIN = new extremeType(3, "Rainiest",_defaultRainUnit);
	extremeType.HSUN = new extremeType(4, "Sunniest",_defaultHoursUnit);

	//PPN Types
	var ppnType = function(val, display){
		ppnType.parentConstructor.call(this, val, display);
	};
	ppnType.inheritsFrom( EnumBase );

	ppnType.RAIN = new ppnType(0, 'Rain');
	ppnType.SNOW = new ppnType(1, 'Snow');
	//TODO: There must be more of these

	var pressureTendency = function(val, display){
		pressureTendency.parentConstructor.call(this, val, display);
	};
	pressureTendency.inheritsFrom( EnumBase );

	pressureTendency.F = new pressureTendency(0, 'Falling');
	pressureTendency.R = new pressureTendency(1, 'Rising');
	pressureTendency.S = new pressureTendency(1, 'Steady');

	// warning safety types.  These are the new NSWWS types.
	var safetyType = function(val, display){
		safetyType.parentConstructor.call(this, val, display);
	};
	safetyType.inheritsFrom( EnumBase );

	safetyType.DRIVING = new safetyType(0, 'Driving');
	safetyType.FLOODING = new safetyType(1, 'Flooding');


	// Warning kinds.  These are the new NSWWS kinds.
	var warningKind = function(val, display){
		warningKind.parentConstructor.call(this, val, display);
	};
	warningKind.inheritsFrom( EnumBase );

	warningKind.ALERT = new warningKind(0, 'Alert');
	warningKind.WARNING = new warningKind(1, 'Warning');


	// warning levels.  These are the new NSWWS types.
	var warningLevel = function(val, display){
		warningLevel.parentConstructor.call(this, val, display);
	};
	warningLevel.inheritsFrom( EnumBase );

	warningLevel.GREEN = new warningLevel(0, 'Green');
	warningLevel.YELLOW = new warningLevel(1, 'Yellow');
	warningLevel.AMBER = new warningLevel(2, 'Amber');
	warningLevel.RED = new warningLevel(3, 'Red');


	// warning statuses.  These are the new NSWWS types.
	var warningStatus = function(val, display){
		warningStatus.parentConstructor.call(this, val, display);
	};
	warningStatus.inheritsFrom( EnumBase );

	warningStatus.ACTIVE = new warningStatus(0, 'Active');
	warningStatus.CANCELLED = new warningStatus(1, 'Cancelled');


	// warning weatherTypes.  These are the new NSWWS types.
	var warningWeatherType = function(val, display){
		warningWeatherType.parentConstructor.call(this, val, display);
	};
	warningWeatherType.inheritsFrom( EnumBase );

	warningWeatherType.SNOW = new warningWeatherType(0, 'Snow');
	warningWeatherType.RAIN = new warningWeatherType(1, 'Rain');
	warningWeatherType.ICE = new warningWeatherType(2, 'Ice');
	warningWeatherType.FOG = new warningWeatherType(3, 'Fog');
	warningWeatherType.WIND = new warningWeatherType(4, 'Wind');

	//Internal Enum
	var blockType = function(val, display){
		blockType.parentConstructor.call(this, val, display);
	};
	blockType.inheritsFrom( EnumBase );
	blockType.DATE = new blockType(0,'Date');


	// ENTITY CLASSES


	/* region{
	 * 	string id <null>
	 *  string name <null>
	 * }
	 */
	var region = buildEntity({id: null, name: null});

	/* extreme
	 * args{
	 *  string type,
	 *  region region,
	 *  number location,
	 *  number value,
	 *  Date issuedAt,
	 *  Date extremeDate
	 * }
	 */
	var extreme = buildEntity({type: null, region:null,location:null,value:null, issuedAt:null, extremeDate:null});

	/* forecast{
	 *	number location <null>,
	 *	forecastDay[] forecastDays <null>,
	 *  Date dataAt <null>
	 * }
	 */
	var forecast = buildEntity({location:null,forecastDays:[],dataAt:null});

	/* forecastDay{
	 *	Date date <null>
	 *	reportValues dayValues <null>
	 *	reportValues nightValues <null>
	 *	reportTime[] forecastTimes <null>
	 * }
	 */
	var forecastDay = 	buildEntity({date:null,dayValues:null,nightValues:null,forecastTimes:null});

	/* forecasterCommentary{
	 *	region region <null>
	 *	string[] report <null>
	 *  Date issuedDate <null>
	 *  string issuedDays <null>
	 * }
	 */
	var forecasterCommentary = buildEntity({region:null,report:null, issuedDate:null, issuedDays:null});

	/* groundConditionsImage{
	 *	string href <null>
	 *	string src <null>
	 *	string title <null>
	 * }
	 */
	var groundConditionsImage = buildEntity({href:null,src:null,title:null});

	/* point{
	 *	number lat <null>
	 *	number lon <null>
	 * }
	 */
	var point = buildEntity({lat:null,lon:null});

	/* locationDetail{
	 *	number altitude <null>
	 *	string regionId <null>
	 *  string nearestObsSiteId
	 * }
	 */
	var locationDetail = buildEntity({
		altitude: null,
		regionId: null,
		nearestObsSiteId: null
	});
	
	/* location{
	 *	number altitude <null>
	 *	number id <null>
	 *	point point <null>
	 *	string name <null>
	 *  region region <null>
	 *  string nationalPark <null>
	 *  string mountainRegion <null>
	 *  boolean showOnMap <null>
	 *  number forecastZoomLevel <null>
	 *  number observationZoomLevel <null>
	 *  string type <null> (forecast Location Type only)
	 * }
	 */
	//TODO Type ENUM
	var location = buildEntity({altitude:null, id:null, point:point, name:null, region:region,
		nationalPark:null, mountainRegion:null, showOnMap:null,
		type:null, forecastZoomLevel:null,observationZoomLevel:null},{
			isInNationalPark : function(){
				return ((typeof this.getNationalPark()) != "undefined" && this.getNationalPark() != null && this.getNationalPark().length > 0);
			},

			isInMountainRegion : function(){
				return ((typeof this.getMountainRegion()) != "undefined" && this.getMountainRegion() != null && this.getMountainRegion().length > 0);
			},

			// All locations are currently forecast locations!
			isForecastLocation : function(){
				return this.getForecastZoomLevel() != null && this.forecastZoomLevel()>0;
			},

			isObservationLocation : function(){
				return this.getObservationZoomLevel() != null && this.getObservationZoomLevel()>0;
			},

			/**
			 * options{
			 *  function success({location:metoffice.location,dist:number})
			 *  function error(message)
			 * }
			 */
			getNearestObservationLocation : function(options) {
				metoffice.location.getNNearest({
					point:this.getPoint(),
					quantity:1,
					observationsOnly:true,
					success:function(list){
						if(list!=null && list.length >0)
						{
							if(_validateExists(options.success))
							{
								options.success(list[0]);
							}
						}
						else{
							if(_validateExists(options.error))
							{
								options.error("No observation sites found");
							}
						}
					},
					error:function(){
						if(_validateExists(options.error))
						{
							options.error("No observation sites found");
						}
					}});
			}
			}
		);

	/* mostSevereWarning{
	 *	warningLevel highest <null>
	 *	warningKind kind <null>
	 *  { warningWeatherType -> mostSevereWarningType } - warningsByType
	 *  mostSevereWarningsByDate warningsByDate <[]>
	 * }
	 */
	var mostSevereWarning = buildEntity({highest:null,kind:null, warningsByType:{}, warningsByDate:[]});

	/* mostSevereWarningType{
	 *  warningsWeatherType type <null>
	 *	warningLevel highest <null>
	 *	warningKind kind <null>
	 *  mostSevereRegionalWarning[] regionalWarnings []
	 * }
	 */
	var mostSevereWarningType = buildEntity({type:null,highest:null,kind:null,regionalWarnings:[]});

	/* mostSevereWarningDate{
	 *  day
	 *  date
	 *	warningLevel highest <null>
	 *	warningKind kind <null>
	 *  { warningWeatherType -> mostSevereWarningType } - warningsByType
	 * }
	 */
	var mostSevereWarningDate = buildEntity({day:null,date:null,highest:null,kind:null,warningsByType:{}});

	/* mostSevereRegionalWarning{
	 * region region <null>
	 * string warningText
     * string forecasterText
	 * warningWeatherType type
	 * date createdDate
	 * date modifiedDate
	 * date validFrom
	 * date validTo
	 * warningStatus status
	 * warningLevel level
	 * warningKind kind
	 * number impact
	 * number likelihood
	 * safetyType[] safetyTypes	// n per warning
	 * string safetyText
	 * point iconPosition
	 * number version
	 * }
	 */
	var mostSevereRegionalWarning = buildEntity({region:null, type:null, warningText: null, forecasterText: null,
		weatherType:null, createdDate: null, modifiedDate: null, validFrom: null,
		validTo: null, status: null, level: null, kind: null, likelihood: null, impact: null,
		safetyTypes: [], safetyText: null, iconPosition: null, version: null});

	/* mountain{
	 *	string id <null>
	 *	string name <null>
	 *	string[] regions <null>
	 *	point point <null>
	 * }
	 */
	//TODO Add Polyline To describe mountain region outline
	var mountain = buildEntity({id:null,name:null,regions:null,point:null});

	/* mountainForecast{
	 *	string mountain <null>
	 *	Date issueDate <null>
	 *	string validFrom <null>
	 *	Date validTo <null>
	 *	string validity <null>
	 *	Date cachedAt <null>
	 *	mountainForecastHazard[] hazards <null>
	 *	string overview <null>
	 *	mountainForecastDay[] forecastDays <null>
	 *	string[] outlooks <null>
	 *	string groundConditionsReport <null>
	 *	groundConditionsImage[] groundConditionsImages <null>
	 *	rainGauge[] rainGauges <null>
	 *  string creatingAuthority <null>
	 *  date creationDate <null>
	 * }
	 */
	var mountainForecast = buildEntity({mountain:null,issueDate:null,validFrom:null,validTo:null,validity:null,
		cachedAt:null,hazards:null,overview:null,forecastDays:null,outlooks:null,
		groundConditionsReport:null, groundConditionsImages:null,rainGauges:null,
		creatingAuthority:null,creationDate:null},{
			hasRainGauges : function() {
				return ((typeof this.getRainGauges()) != "undefined"
						&& this.getRainGauges() != null && this.getRainGauges().length > 0);
			},
			hasGroundConditionsReport : function() {
				return ((typeof this.getGroundConditionsReport()) != "undefined"
						&& this.getGroundConditionsReport() != null && this.getGroundConditionsReport().length > 0);
			},
			hasGroundConditionsImages : function() {
				return ((typeof this.getGroundCondtionsImages()) != "undefined"
						&& this.getGroundCondtionsImages() != null && this.getGroundCondtionsImages().length > 0);
			}
		}
	);


	/* mountainForecastDay{
	 *	string weather <null>
	 *	string visibility <null>
	 *	string hillFog <null>
	 *	string maxWindLevel <null>
	 *	string maxWind <null>
	 *	string tempLowLevel <null>
	 *	string tempHighLevel <null>
	 *	string freezingLevel <null>
	 *	mountainWeatherPeriod[] weatherPeriods <null>
	 * }
	 */
	var mountainForecastDay = buildEntity({weather:null,visibility:null,hillFog:null,maxWindLevel:null,maxWind:null,
		tempLowLevel:null,tempHighLevel:null,freezingLevel:null,weatherPeriods:null});

	/* mountainForecastHazard{
	 *	string type <null>
	 *	string risk <null>
	 *	string comment <null>
	 * }
	 */
	var mountainForecastHazard = buildEntity({type:null,risk:null,comment:null});

	/* mountainWeatherPeriod{
	 *	string timeSpan <null>
	 *	number weather <null>
	 *	number probability <null>
	 *	ppnType ppnType <null>
	 * }
	 */
	var mountainWeatherPeriod = buildEntity({timeSpan:null,weather:null,probability:null,ppnType:null});

	/* nationalPark
	 * 	string id <null>
	 *  string name <null>
	 *  string[] regions <[]>
	 */
	var nationalPark = buildEntity({id: null, name: null, regions:[]});

	/* nationalParkForecast
	 * args{
	 * string nationalParkId <null>
	 * Date issuedAt <null>
	 * string issuedBy <null>
	 * nationalParkForecastSection[] sections <null>
	 * }
	 */
	var nationalParkForecast = buildEntity({nationalParkId: null, issuedAt: null,issuedBy: null, sections:null});

	/* nationalParkForecastSection
	 * 	string id <null>
	 *  string title <null>
	 *  string text <null>
	 */
	var nationalParkForecastSection = buildEntity({id: null, title: null, text: null});

	/* observation{
	 * int location <null>
	 * observationDay[] observationDays <null>
	 * Date issuedAt <null>
	 * }
	 */
	var observation = buildEntity({location:null,observationDays:[], issuedAt:null});

	/* observationDay{
	 *	Date date <null>
	 *	reportTime[] observationTimes <null>
	 * }
	 */
	var observationDay = buildEntity({date:null,observationTimes:[]});

	/* rainGauge{
	 *	string type <null>
	 *  string site <null>
	 *  string location <null>
	 *  number altitude <null>
	 *  number wmo <null>
	 *  rainGaugeReading[] readings <null>
	 *  Date measurementDate
	 * }
	 */
	var rainGauge = buildEntity({type:null,site:null,altitude:null,location:null,wmo:null,
		readings:null,measurementDate:null});

	/* rainGaugeReading{
	 *	number hours <null>
	 *	number reading <null>
	 * }
	 */
	var rainGaugeReading =  buildEntity({hours:null,reading:null});




	/* regionalForecast{
	 *	Date createdOn <null>
	 *	Date issuedAt <null>
	 *	region region <null>
	 *	regionalForecastSection[] day1To2 <null>
	 *	regionalForecastSection[] day3To5 <null>
	 *	regionalForecastSection[] day6To15 <null>
	 * }
	 */
	var regionalForecast = buildEntity({createdOn:null,issuedAt:null,region:null,day1To2:null,
		day3To5:null,day6To15:null});

	/* regionalForecastSection{
	 *	string title <null>
	 *	string description <null>
	 * }
	 */
	var regionalForecastSection = buildEntity({title:null,description:null});

	/* reportTime{
	 *	Date time <null>
	 *  reportValue[] reportValues <null>
	 * }
	 */
	var reportTime = buildEntity({time:null, reportValues:null});

	/* reportValues{
	 *	number weather <null>
	 *	String windDirection <null>
	 *  number windSpeed <null>
	 *  number windGust <null>
	 *  number visibility <null>
	 *  number temperature <null>
	 *  number feelsLikeTemperature <null>
	 *  number uv <null>
	 *  number humidity <null>
	 *  number pressure <null>
	 *  pressureTendency pressureTendency <null>
	 * }
	 */
	var reportValues = buildEntity({weather:null, windDirection:null,windSpeed:null,windGust:null,
			visibility:null, temperature:null,feelsLikeTemperature:null,
			pressure:null,pressureTendency:null,uv:null,humidity:null});

	/*
	 * Timezone
	 *   string id <null>
	 *   number offset <null>
	 */
	var timezone = buildEntity({id:null,offset:null});

	/*
	warning
		id id
		string warningText
		string forecasterText
		warningWeatherType weather // one per warning
		date createdDate
		date modifiedDate
		date validFrom
		date validTo
		warningStatus status
		warningLevel level
		warningKind kind
		number impact
		number likelihood
		safetyType[] safetyTypes	// n per warning
		string safetyText
		point iconPosition
		number version

		zone[] _zones
	*/
	var warning = buildEntity({
		id: null, warningText: null, forecasterText: null, weatherType:null, createdDate: null, modifiedDate: null, validFrom: null,
		validTo: null, status: null, level: null, kind: null, likelihood: null, impact: null, safetyTypes: [], safetyText: null, iconPosition: null, version: null,
		zones: []},
		{
			/*
			 * Checks whether the warning encapsulates the specified region
			 * within any of its zones
			 */
			includesRegion : function(testRegion) {
				for (var i = 0; i < this.getZones().length; i++) {
					if (this.getZones()[i].includesRegion(testRegion)) return true;
				}
				return false;
			},
			/*
			 * Returns the highest warningLevel from this warning for the specified region
			 * or null if the region does not include the region
			 */
			getMaxLevelByRegion : function(testRegion) {
				var result = null;
				for (var i = 0; i < this.getZones().length; i++) {
					if (this.getZones()[i].includesRegion(testRegion)) {
						var nextValue = this.getZones()[i].getLevel();
						if (result == null || (nextValue.getValue() > result.getValue())) {
							result = nextValue;
						}
					}
				}
				return result;
			},
			/*
			 * Returns all the warningRegions associated with the zones of this warning
			 * */
			getRegions : function() {
				var result = [];
				var zones = this.getZones();
				for (var i = 0; i < zones.length; i++) {
					var thisZone = zones[i];
					var zoneRegions = thisZone.getRegions();
					for (var j = 0; j < zoneRegions.length; j++) {
						if (! utility.arrayContains(result, zoneRegions[j] )) {
							result.push(zoneRegions[j]);
						}
					}
				}
				return result;
			},
			/*
			 *  For all the regions referenced by this warning, this method will find the highest
			 *  warningLevel for each of the regions referenced across the zones of this warning.
			 *  These are all returned in an array of warningRegion->warningLevel object literals
			 */
			getMaxLevelByAllRegions : function() {
				var result = [];
				var regions = this.getRegions();
				var rlen = regions.length;
				for (var i = 0; i < rlen; i++) {
					var thisResult = {region: regions[i], level: getMaxLevelByRegion(regions[i])};
					result.push(thisResult);
				}
				return result;
			}
		}
	);

	/*
	zone
		id _id
		point[] _polygon
		warningRegion[] _warningRegions
		warningLevel _level
		int _likelihood
		int _impact
	*/
	var zone = buildEntity({id: null, polygon:[], warningRegions: [], level: null, likelihood:null, impact:null},
		{
			getRegions : function() {
				var result = [];
				var regions = this.getWarningRegions();
				var rlen = regions.length;
				for (var i =0; i < rlen; i++) {
					var thisRegion = regions[i];
					if (! utility.arrayContains(result, thisRegion.getRegion())) {
						result.push(thisRegion.getRegion());
					}
				}
				return result;
			},

			includesRegion : function(testRegion) {
				testRegion = testRegion.toUpperCase();
				for (var i = 0; i < this.getWarningRegions().length; i++) {
					if (this.getWarningRegions()[i].getRegion().getId().toUpperCase() == testRegion) return true;
				}
				return false;
			}
		}
	);

	/* warningRegion {
		region _region			// region
		string[] _subRegions	// list of unitary authority names
		}
	*/
	var warningRegion = buildEntity({region:null,subRegions:[]});

	/* weatherwatch{
	 *	string headline <null>
	 *	string text <null>
	 *	weatherwatchURL[] urls <null>
	 * }
	 */
	var weatherwatch = buildEntity({headline:null,text:null,urls:null});

	/* weatherwatchURL{
	 *	string title <null>
	 *	string href <null>
	 * }
	 */
	var weatherwatchURL = buildEntity({title:null,href:null});

	//
	var _mountains = {
			yorkshiredales: new mountain({id:"yorkshiredales", name:"Yorkshire Dales", regions:["yh"], point:new point({lat:"54.21",lon:"-2.17"})}),
			westhighland: new mountain({id:"westhighland", name:"West Highland", regions:["st","he","ta"], point:new point({lat:"56.84",lon:"-4.76"})}),
			snowdonia: new mountain({id:"snowdonia", name:"Snowdonia", regions:["wl"], point:new point({lat:"52.83",lon:"-3.94"})}),
			peakdistrict: new mountain({id:"peakdistrict", name:"Peak District", regions:["nw","yh","wm","em"], point:new point({lat:"53.33",lon:"-1.86"})}),
			lakedistrict: new mountain({id:"lakedistrict", name:"Lake District", regions:["nw"], point:new point({lat:"54.42",lon:"-3.09"})}),
			easthighland: new mountain({id:"easthighland", name:"East Highland", regions:["gr","he","ta"], point:new point({lat:"56.82",lon:"-3.41"})}),
			breconbeacons: new mountain({id:"breconbeacons", name:"Brecon Beacons", regions:["wl","sw"], point:new point({lat:"51.78",lon:"-3.28"})})
		};

	var _regions = {
		uk: new region({id:"uk", name:"UK"}),
		os: new region({id:"os", name:"Orkney & Shetland"}),
		he: new region({id:"he", name:"Highland & Eilean Siar"}),
		gr: new region({id:"gr", name:"Grampian"}),
		st: new region({id:"st", name:"Strathclyde"}),
		ta: new region({id:"ta", name:"Central, Tayside & Fife"}),
		dg: new region({id:"dg", name:"Dumfries"}),
		ni: new region({id:"ni", name:"Northern Ireland"}),
		wl: new region({id:"wl", name:"Wales"}),
		nw: new region({id:"nw", name:"North West England"}),
		ne: new region({id:"ne", name:"North East England"}),
		yh: new region({id:"yh", name:"Yorkshire & Humber"}),
		wm: new region({id:"wm", name:"West Midlands"}),
		em: new region({id:"em", name:"East Midlands"}),
		ee: new region({id:"ee", name:"East of England"}),
		sw: new region({id:"sw", name:"South West England"}),
		se: new region({id:"se", name:"London & South East England"})
	};

	//STATIC METHODS

	//EXTREME STATICS
	var parseAllExtremes = function(data, options) {
		var results = {};
		var issuedAt = utility.ISO8601toDate($(data).find('UkExtremes').attr('issuedAt'));
		var extremeDate = utility.ISO8601toDate($(data).find('UkExtremes').attr('extremeDate'));
		$(data).find("Region").each(function(){
			var type = _parseString(this,'type', true);
			type = (type!=null)?extremeType.parse(type):null;
			var uom = null;
			if(type != null){
				uom = type.getDefaultUom();
			}
			var regionId = _parseString(this,'id', true);
			var regionName = _parseString(this,'name', true);
			$(this).find('Extreme').each(function(){
				var myExtreme = new extreme({
					type:extremeType.parse(_parseString(this,'type', true)),
					region:utility.getRegionFromId(regionId),
					location:_parseNumber(this,'locId', true),
					value:_parseMeasurement(this, uom),
					issuedAt:issuedAt,
					extremeDate:extremeDate
				});

				if (!results[regionId]) {
					results[regionId] = [];
				}
				results[regionId].push(myExtreme);
			});
		});

		for (var j in results) {
			cache.put(_extremeCacheKey+"/" + j, results[j], options);
		}

		return results;
	};

	/*
	 * metoffice.extreme.getByRegion(
	 * options{
	 * string region
	 * function success(metoffice.extreme[]) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options specified
	 * */
	extreme.getByRegion = function(options) {
		options = (_validateExists(options))?options:{};
		if(!_validatePopulated(options.region))
		{
			throw "No region parameter specified (extreme.getByRegion)";
		}
		var region = options.region;

		_getData({
			cacheKey: _extremeCacheKey+"/" + region,
			dataKey: _extremeCacheKey+"/"+"All",
			requestKey: _extremeCacheKey+"/"+"Request",
			url: _extremeURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: parseAllExtremes,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: options.ident,
			waitType:blockType.DATE
		});
	};

	/*
	 * metoffice.extreme.getAllRegions(
	 * options{
	 * function success({regionIds:extreme[]}) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options specified
	 * */
	extreme.getAllRegions = function(options){
		options= (_validateExists(options))?options:{};

		_getData({
			cacheKey:_extremeCacheKey+"/All",
			requestKey:_extremeCacheKey+"/Request",
			url:_extremeURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseAllExtremes,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:"all",
			waitType:blockType.DATE
		});

	};

	//FORECAST STATICS
	/*
	 * metoffice.forecast.getById(
	 * options{
	 * number id,
	 * function success(metoffice.forecast) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.id specified
	 * */
	forecast.getById = function(options){
		options = (_validateExists(options))?options:{};
		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (forecast.getById)";
		}
		if (typeof(options.id) != "number" ) {
			throw "The id parameter must be a number (forecast.getById)";
		}

		var id = options.id;

		_getData({
			cacheKey:_forecastCacheKey+"/" + id,
			url:_forecastURL+id,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:createForecast,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:id,
			waitType:blockType.DATE
		});

		function createForecastReport(values)
		{
            return new metoffice.forecast({
				location:values.location,
				forecastDays:values.days,
				dataAt:utility.ISO8601toDate($(values.report).parent().attr('dataDate'))
			});
		}

		function createForecastDaySection(values)
		{
			return new metoffice.forecastDay({
				 date:values.date,
				 dayValues:_parseReportValues($(values.day).find('DayValues'),values.units),
				 nightValues:_parseReportValues( $(values.day).find('NightValues'),values.units),
				 forecastTimes:values.times
			});
		}
		function createForecast(data) {
			return _parseLocationReport(data, createForecastReport,createForecastDaySection);
		}
	};

	//FORECASTER COMMENTARY STATICS
	/*
	 * metoffice.forecasterCommentary.getAllRegions(
	 * options{
	 * function success(metoffice.forecasterCommentary[]) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options specified
	 * */
	forecasterCommentary.getAllRegions = function(options){
		options= (_validateExists(options))?options:{};

		_getData({
			cacheKey:_forecasterCommentaryCacheKey+"/All",
			requestKey:_forecasterCommentaryCacheKey+"/Request",
			url:_forecasterCommentaryURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseForecastCommentariesAll,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:"all"
		});

		function parseForecastCommentariesAll(data, options) {
			var commentaries=[];
			$(data).find('report').each(function(){
				 var issuedDate = _parseString($(this).parent().find('issue'),'at', true);
				 var issuedDays = _parseString($(this).parent().find('issue'),'day', true);
				 var regions = [];
				 $(this).find('region').each(function(){
					 var paras = [];

					 $(this).find('para').each(function(){
						 paras.push($(this).text());
					 });

					 var regionCommentary = new metoffice.forecasterCommentary({
						 region:utility.getRegionFromId(_parseString(this,'id', true)),
						 issuedDate:issuedDate,
						 issuedDays:issuedDays,
						 report:paras
					 });

					 cache.put(_forecasterCommentaryCacheKey+"/"+regionCommentary.getRegion().getId(),regionCommentary, options);

					 commentaries.push( regionCommentary );
				 });
			});

			return commentaries;
		}
	};

	/*
	 * metoffice.forecasterCommentary.getByRegion(
	 * options{
	 * string region,
	 * function success(metoffice.forecast) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options specified
	 * */
	forecasterCommentary.getByRegion = function(options){
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.region))
		{
			throw "No region parameter specified (forecasterCommentary.getByRegion)";
		}
		var region = options.region;

		_getData({
			cacheKey:_forecasterCommentaryCacheKey+"/"+region,
			requestKey:_forecasterCommentaryCacheKey+"/Request",
			url:_forecasterCommentaryURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseForecastCommentaryRegion,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:options.region
		});

		function parseForecastCommentaryRegion(data) {
			var regionCommentary;
			$(data).find('report').each(function(){
				 var issuedDate = $(this).parent().find('issue').attr('at');
				 var issuedDays = $(this).parent().find('issue').attr('day');
				 $(this).find('region[id="'+region+'"]').each(function(){
					 var paras = [];
					 var regionId = $(this).attr('id');
					 $(this).find('para').each(function(){
						 paras.push($(this).text());
					 });

					 regionCommentary = new metoffice.forecasterCommentary({
						 region:utility.getRegionFromId(regionId),
						 issuedDate:issuedDate,
						 issuedDays:issuedDays,
						 report:paras
					 });
				 });
			});

			return regionCommentary;
		}
	};

	//LOCATION STATICS
	var parseLocation = function(xLocation)
	{
 var r = utility.getRegionFromId(_parseString(xLocation,"r",true));
		return new location({
				altitude: _parseNumber(xLocation, "a", true) || null,//Several of the values below have an OR options as well (null), 
				id: _parseNumber(xLocation, "id", true),			//to be left in whilst checking older applications do not still need them (since change of feed)
				point: {
					lat: _parseNumber(xLocation, "latitude", true),
					lon: _parseNumber(xLocation, "longitude", true)
				},
				name: _parseString(xLocation, "name", true),
				region: r ? r.values() : null,
				forecastZoomLevel: _parseNumber(xLocation, "zoomLevel", true),
				nationalPark: _parseString(xLocation, "p", true) || null,
				mountainRegion: _parseString(xLocation, "m", true) || null,
				showOnMap: _parseNumber(xLocation, "s", true) || 1,
				type: _parseString(xLocation, "type", true),
				observationZoomLevel: _parseNumber(xLocation, "obsZoomLevel", true)
/*		
		altitude:_parseNumber(xLocation,"a",true),
			id:_parseNumber(xLocation,"id",true),
			point: {lat: _parseNumber(xLocation,"l",true), lon: _parseNumber(xLocation,"g",true)},
			name: _parseString(xLocation,"n",true),
			region: r ? r.values() : null,
			forecastZoomLevel:_parseNumber(xLocation,"z",true),
			nationalPark:_parseString(xLocation,"p",true),
			mountainRegion:_parseString(xLocation,"m",true),
			showOnMap:_parseNumber(xLocation,"s",true),
			type:_parseString(xLocation,"t",true),
			observationZoomLevel: _parseNumber(xLocation,"o",true)
*/
			});
	};


	/*
	 * metoffice.location.getById()
	 *
	 */
	location.getById = function (options) {
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (location.getById)";
		}
		if (typeof(options.id) != "number" ) {
			throw "The id parameter must be of type number (location.getById)";
		}

		var id = options.id;

		_getData({
			cacheKey: _locationCacheKey+"/" + id,
			requestKey: _locationCacheKey+"/Request",
			url: _locationURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: parseLocations,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: id
		});

		function parseLocations(data) {
			var oLocation = null;
			$(data).find("x[i=" + id + "]").each(function(){
				oLocation =  parseLocation(this);
			});

			return oLocation;
		}
	};

	/*
	 * metoffice.location.getAll(
	 * options{
	 * function success(metoffice.location[]) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options specified
	 * */
	location.getAll = function(options){
		options= (_validateExists(options))?options:{};
		_getData({
			cacheKey:_locationCacheKey+"/All",
			requestKey:_locationCacheKey+"/Request",
			url:_locationURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseAllLocations,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:"all"
		});

		function parseAllLocations(data, options) {
		var locations=[];
            $(data).find("Location").each(function(){
				var oLocation =  parseLocation(this);
				cache.put(_locationCacheKey+"/" + oLocation.getId(),oLocation);
				locations.push(oLocation);
			});
			return locations;
		}
	};

	/*
	 * metoffice.locationDetail.getById()
	 *
	 */
	locationDetail.getById = function (options) {
		options = (_validateExists(options)) ? options : {};

		if (!_validatePopulated(options.id)) {
			throw "No id parameter specified (location.getById)";
		}
		if (typeof (options.id) != "number") {
			throw "The id parameter must be of type number (location.getById)";
		}

		var id = options.id;

		_getData({
			cacheKey: _locationDetailCacheKey + "/" + id,
			requestKey: _locationDetailCacheKey + "/Request",
			url: _locationDetailURL + id,
			dataType: "xml",
			timeToLive: options.timeToLive,
			timeout: options.timeout,
			onDataCallback: parseLocationDetail,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: id,
			async: options.async
		});

		function parseLocationDetail(data) {
			var locDetailXml = $(data).find("LocationDetail")[0];
			locDetail = new locationDetail({
				altitude: _parseString(locDetailXml, "altitude", true),
				regionId: _parseString(locDetailXml, "region", true),
				nearestObsSiteId: _parseString(locDetailXml, "nearestObsSite", true)
			});
			return locDetail;
		}
	};
	
	/*
	 * args{
	 * 	metoffice.point point,
	 *  int quantity,
	 *  boolean observationsOnly,
	 *  function success (array of {location:metoffice.location,dist:number}),
	 *  function error
	 * }
	 * */
	location.getNNearest = function(options) {

		function processSuccessCallback(data,processingTime){
			if(_validateExists(options.success)){
				options.success(data,processingTime);
			}
		}
		function processErrorCallback(error){
			if(_validateExists(options.error)){
				options.error(error);
			}
		}
        metoffice.location.getAll({
            success:function(list){
			var start = new Date().getTime(), nearestSites = [], Rad = 6371, radian = Math.PI/180, newDist, a, siteList = [],
                        i, len = list.length, val, valLat, valLon, point,
                    lat = options.point.getLat(),
                    lng = options.point.getLon(),
                    n = options.quantity,
                    obsOnly = !!options.observationsOnly;

                function sortDists(a,b) {
                     return a.dist - b.dist;
                }
                for( i = 0 ; i < len; i ++ ){
                    val = list[i];
                    if(!obsOnly || val.isObservationLocation()) {
                        point = val.getPoint();
                        valLat = point.getLat();
                        valLon = point.getLon();
                        newDist=999999999;
                        a = Math.pow(Math.sin(((lat-valLat)* radian)/2),2) + Math.cos(valLat*radian) * Math.cos(lat*radian) * Math.pow(Math.sin(((lng-valLon)* radian)/2),2);
                        newDist = Rad * (2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)));
                        siteList.push({dist:newDist,location:val});
                    }
                }
                siteList.sort(sortDists);
				for (i = 0; i < n; i++) {
                    nearestSites.push(siteList[i]);
                }
                processSuccessCallback(nearestSites, new Date().getTime() - start);
            },
            error:function(){
                processErrorCallback("Failed to load locations list");
            }
		});
	};

	//MOUNTAIN STATICS
	/*
	 * metoffice.mountain.getAll(
	 * options{
	 * function success(metoffice.mountain[]) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options parameter specified
	 * */
	mountain.getAll = function(options) {
		options= (_validateExists(options))?options:{};

		var results = [];
		$.each(_mountains, function(i,mountain){
			results.push(mountain);
		});
		if(_validateExists(options.success))
		{
			options.success(results);
		}
	};

	/*
	 * metoffice.mountain.getById(
	 * options{
	 * string id,
	 * function success(metoffice.mountain) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.id specified
	 * */
	mountain.getById = function(options) {
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (mountain.getById)";
		}

		if (_mountains[options.id]) {
			if(_validateExists(options.success))
			{
				options.success(_mountains[options.id]);
			}
		} else {
			if(_validateExists(options.error))
			{
				options.error("No match found");
			}
		}
	};

	/*
	 * metoffice.mountain.getByName(
	 * options{
	 * string name,
	 * function success(metoffice.mountain) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.name specified
	 * */
	mountain.getByName = function(options){
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.name))
		{
			throw "No name parameter specified (mountain.getByName)";
		}

		for (var m in _mountains) {
			if(m && _mountains[m] && _mountains[m].getName()===options.name){
				if(_validateExists(options.success))
				{
					options.success(_mountains[m]);
				}
				return;
			}
		}

		if(_validateExists(options.error))
		{
			options.error("No match found");
		}
	};

	//MOUNTAIN FORECAST STATICS
	/*
	 * metoffice.mountainForecast.getById(
	 * options{
	 * string id,
	 * function success(metoffice.mountainForecast) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.id specified
	 * */
	mountainForecast.getById = function(options){
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (mountainForecast.getById)";
		}
		//The query is actually done based on mountain name with escaped spaces.
		var id = options.id;
		if(_mountains[id])
		{
			id=_mountains[id].getName();
		}

		_getData({
			cacheKey:_mountainForecastCacheKey+"/" + id,
			url:_mountainForecastURL+id,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseMountainForecast,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:id,
			waitType:blockType.DATE
		});

		function parseMountainForecast(data) {
			var mountainForecast;
			$(data).find('report').each(function(){
				 var hazards = [];
				 $(this).find('Hazard').each(function(){
					 hazards.push( new metoffice.mountainForecastHazard({
						type:_parseString(this,'Element'),
						risk:_parseString(this,'Risk'),
						comment:_parseString(this,'Comments')
					}));
				 });

				 var forecastDays = [];
				 $(this).find('ForecastDay0, ForecastDay1').each(function(){
					 periods = [];
					 $(this).find('WxPeriod').each(function(){

						 periods.push( new metoffice.mountainWeatherPeriod({
							timeSpan:_parseString(this,'Period'),
							weather:_parseNumber(this,'Weather'),
							probability:_parseString(this,'Probability'),
							ppnType:ppnType.parse(_parseString(this,'Ppn_type'))
						}));
					 });

					 forecastDays.push( new metoffice.mountainForecastDay({
						weather:_parseString(this,'Weather'),
						visibility:_parseString(this,'Visibility'),
						hillFog:_parseString(this,'HillFog'),
						maxWindLevel:_parseString(this,'MaxWindLevel'),
						maxWind:_parseString(this,'MaxWind'),
						tempLowLevel:_parseMeasurement(this, 'TempLowLevel',_defaultTempUnit),
						tempHighLevel:_parseMeasurement(this, 'TempHighLevel',_defaultTempUnit),
						freezingLevel:_parseString(this,'FreezingLevel'),
						weatherPeriods:periods
					}));
				 });
				 var outlooks = [];
				 $(this).find('OutlookDay2,OutlookDay3,OutlookDay4').each(function(){
					 outlooks.push($(this).text());
				 });

				 var images = [];
				 $(this).find('Image').each(function(){
					 images.push(new metoffice.groundConditionsImage({
						 href: _parseString(this,'href',true),
						 src: _parseString(this,'src',true),
						 title: _parseString(this,'title',true)
					}));
				 });

				 var rainGauges = [];
				 $(this).find('RainGauge').each(function(){
					 var readings = [];
					 $(this).find('Text').each(function(){
						 readings.push(new rainGaugeReading({
							 hours: _parseNumber(this,'hours',true),
							 reading:_parseNumber(this)
						}));
					 });

					 rainGauges.push(new rainGauge({
						 type:_parseString(this,'type',true),
						 measurementDate: utility.ISO8601toDate($(this).attr('measurementDate')),
						 site:_parseString(this,'Site'),
						 altitude:_parseMeasurement(this, 'Altitude',_defaultDistanceUnit),
						 location:_parseString(this,'Location'),
						 wmo:_parseString(this,'WMO'),
						 readings:readings
					}));
				 });
				var issueDate="";
				$(this).find('issue').each(function(){
					issueDate+=$(this).attr('date');
					issueDate+='T';
					var time = _parseString(this,'time',true);
					if(time != null && time.length ==4){
						time=time.substring(0,2)+":"+time.substring(2)+"z";
					}
					else{
						time="00:00z";
					}
					issueDate+=time;
				});

				mountainForecast= new metoffice.mountainForecast({
					mountain:_parseString(this,'location'),
					issueDate:utility.ISO8601toDate(issueDate),
					validFrom:utility.ISO8601toDate(_parseString(this,'ValidFrom')),
					validTo:utility.ISO8601toDate(_parseString(this,'ValidTo')),
					validity:_parseString(this,'Validity'),
					cachedAt:_parseString(this,'CachedAt'),
					hazards:hazards,
					overview:_parseString(this,'Overview'),
					forecastDays:forecastDays,
					outlooks:outlooks,
					groundConditionsReport:_parseString($(this).find('GroundConditions'),'Text'),
					groundConditionsImages:images,
					rainGauges:rainGauges,
					creatingAuthority:_parseString(this,'creating-authority',true),
					creationDate:utility.ISO8601toDate(_parseString(this,'creation-time',true))
				});
			});

			return mountainForecast;
		}
	};

	//NATIONAL PARKS STATICS
	var _parseAllNationalParkForecasts = function(data, options) {
		var byRegion = {};
		var allForecasts = {};
		var allParks = {};
		$(data).find("NationalPark").each(function(){
			var regions = [];
			$(this).find('Region').each(function(){
				var reg = _parseString(this,'id',true);
				if(reg != null)
				{
					regions.push(reg);
				}
			});

			var park = new nationalPark({
				id:_parseString(this,'id', true),
				name:_parseString(this,'name', true),
				regions:regions});

			//Cache NationalParks
			cache.put(_nationalParkCacheKey+"/"+park.getId(), park, options);
			allParks[park.getId()] = park;

			var issuedBy = _parseString(this,'issuedBy', true);
			var issuedAt = utility.ISO8601toDate($(this).attr('issuedAt'));

			var sections = [];

			$(this).find('Period').each(function(){
				 sections.push( new nationalParkForecastSection({
						id:_parseString(this,'id',true),
						title:_parseString(this,'Title'),
						text:_parseString(this,'Text')
					}));
			});
			var parkForecast = new nationalParkForecast({
				nationalParkId:park.getId(),
				issuedAt:issuedAt,
				issuedBy:issuedBy,
				sections:sections
			});

			//Forecasts by national park
			cache.put(_nationalParkForecastCacheKey+"/"+parkForecast.getNationalParkId(), parkForecast, options);

			//All National Park Forecasts
			allForecasts[parkForecast.getNationalParkId()]=parkForecast;

			//Forecasts by region
			for(var i = 0 ; i < park.getRegions().length; i++)
			{
				var reg = park.getRegions()[i];
				if (!byRegion[reg.getId()]) {
					byRegion[reg.getId()] = {};
				}
				by = byRegion[reg.getId()];
				by[parkForecast.getNationalParkId()]=parkForecast;
			}
		});

		for (var j in byRegion) {
			//Cache forecasts by region
			cache.put(_nationalParkForecastCacheKey+"/" + j, byRegion[j], options);
		}

		cache.put(_nationalParkCacheKey+"/" + "All", allParks, options);

		return allForecasts;
	};

	/*
	 * gets a nationalPark given an Id.
	 *
	 * metoffice.nationalPark.getById(
	 * options{
	 * string id <Exmoor, Dartmoor etc>,
	 * function success(metoffice.nationalPark) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options.id specified
	 *
	 */
	nationalPark.getById = function(options) {
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No region parameter specified (nationalParkForecast.getById)";
		}

		var id = options.id;

		_getData({
			cacheKey: _nationalParkCacheKey+"/" + id,
			dataKey: _nationalParkForecastCacheKey+"/"+"All",
			requestKey: _nationalParkForecastCacheKey+"/"+"Request",
			url: _nationalParkForecastRegionURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: _parseAllNationalParkForecasts,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: options.ident
		});
	};

	/*
	 * gets a list of all national parks.
	 *
	 * metoffice.nationalPark.getAll(
	 * options{
	 * function success(metoffice.nationalPark[]) <optional>,
	 * function error(errorString) <optional>)
	 *
	 */
	nationalPark.getAll = function(options) {
		options= (_validateExists(options))?options:{};

		_getData({
			cacheKey: _nationalParkCacheKey+"/" + "All",
			dataKey: _nationalParkForecastCacheKey+"/"+"All",
			requestKey: _nationalParkForecastCacheKey+"/"+"Request",
			url: _nationalParkForecastRegionURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: _parseAllNationalParkForecasts,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: options.ident
		});
	};

	//NATIONAL PARKS FORECAST STATICS
	/*
	 * gets a set of nationalParkForecasts given a region Id.
	 *
	 * metoffice.nationalParkForecast.getByRegion(
	 * options{
	 * string region <sw,nw etc>,
	 * function success({id:metoffice.nationalParkForecast}) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.region specified
	 *
	 */
	nationalParkForecast.getByRegion = function(options) {
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.region))
		{
			throw "No region parameter specified (nationalParkForecast.getByRegion)";
		}

		var region = options.region;

		_getData({
			cacheKey: _nationalParkForecastCacheKey+"/" + region,
			dataKey: _nationalParkForecastCacheKey+"/"+"All",
			requestKey: _nationalParkForecastCacheKey+"/"+"Request",
			url: _nationalParkForecastRegionURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: _parseAllNationalParkForecasts,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: options.ident,
			waitType:blockType.DATE
		});
	};

	/*
	 * gets a  nationalParkForecasts given a park Id.
	 *
	 * metoffice.nationalParkForecast.getByParkId(
	 * options{
	 * string park <dartmoor, exmoor>,
	 * function success(metoffice.nationalParkForecast) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options.park specified
	 *
	 */
	nationalParkForecast.getByPark = function(options) {
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.park))
		{
			throw "No park parameter specified (nationalParkForecast.getByPark)";
		}

		var park = options.park;

		_getData({
			cacheKey: _nationalParkForecastCacheKey+"/" + park,
			dataKey: _nationalParkForecastCacheKey+"/"+"All",
			requestKey: _nationalParkForecastCacheKey+"/"+"Request",
			url: _nationalParkForecastRegionURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: _parseAllNationalParkForecasts,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: options.ident,
			waitType:blockType.DATE
		});
	};

	/*
	 * gets a set of all nationalParkForecasts.
	 *
	 * metoffice.nationalParkForecast.getAll(
	 * options{
	 * function success({id:metoffice.nationalParkForecast}) <optional>,
	 * function error(errorString) <optional>)
	 */
	nationalParkForecast.getAll = function(options) {
		options= (_validateExists(options))?options:{};

		_getData({
			cacheKey: _nationalParkForecastCacheKey+"/" + "All",
			requestKey: _nationalParkForecastCacheKey+"/"+"Request",
			url: _nationalParkForecastRegionURL,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback: _parseAllNationalParkForecasts,
			onSuccessCallback: options.success,
			onErrorCallback: options.error,
			ident: options.ident,
			waitType:blockType.DATE
		});
	};

	//OBSERVATION STATICS
	/*
	 * metoffice.observation.getById(
	 * options{
	 * number id,
	 * function success(metoffice.observation) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.id specified
	 * */
	observation.getById = function(options){
		options = (_validateExists(options))?options:{};
		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (observation.getById)";
		}
		if (typeof(options.id) != "number" ) {
			throw "The id parameter must be a number (observation.getById)";
		}
		var id = options.id;

		_getData({
			cacheKey:_observationCacheKey+"/" + id,
			url:_observationURL+id,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:createObservation,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:id,
			waitType:blockType.DATE
		});

		function createObservationReport(values)
		{
			return new metoffice.observation({
				location:values.location,
				observationDays:values.days,
				issuedAt:utility.ISO8601toDate($(values.report).parent().attr('issueDate'))
			});
		}

		function createObservationDaySection(values)
		{
			return new metoffice.observationDay({
				 date:values.date,
				 observationTimes:values.times
			});
		}
		function createObservation(data) {
			return _parseLocationReport(data, createObservationReport,createObservationDaySection);
		}
	};

	//REGION STATICS
	/*
	 * metoffice.region.getAll(
	 * options{
	 * function success(metoffice.region[]) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options parameter specified
	 * */
	region.getAll = function(options) {
		options= (_validateExists(options))?options:{};

		var results = [];
		$.each(_regions, function(i,region){
			results.push(region);
		});
		if(_validateExists(options.success))
		{
			options.success(results);
		}
	};

	/*
	 * metoffice.region.getById(
	 * options{
	 * string id,
	 * function success(metoffice.region) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.region specified
	 * */
	region.getById = function(options) {
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (region.getById)";
		}
		// gets a region object, givn a region id.
		if (_regions[options.id]) {
			if(_validateExists(options.success))
			{
				options.success(_regions[options.id]);
			}
		} else {
			if(_validateExists(options.error))
			{
				options.error("No match found");
			}
		}
	};

	/*
	 * metoffice.region.getByName(
	 * options{
	 * string name,
	 * function success(metoffice.region) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options.name specified
	 * */
	region.getByName = function(options){
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.name))
		{
			throw "No name parameter specified (region.getByName)";
		}

		for (var m in _regions) {
			if(m && _regions[m] && _regions[m].getName()===options.name){
				if(_validateExists(options.success))
				{
					options.success(_regions[m]);
				}
				return;
			}
		}

		if(_validateExists(options.error))
		{
			options.error("No match found");
		}
	};

	//REGIONAL FORECAST STATICS
	/*
	 * metoffice.regionalForecast.getByRegion(
	 * options{
	 * string region,
	 * function success(metoffice.regionalForecast) <optional>,
	 * function error(errorString) <optional>)
	 *
	 * throws "Parameter validation failed" if no options or options.region specified
	 * */
	regionalForecast.getByRegion = function(options){
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.region))
		{
			throw "No region parameter specified (regionalForecast.getByRegion)";
		}
		//The query is actually done based on region name with escaped spaces.
		region = options.region;

		_getData({
			cacheKey:_regionalForecastCacheKey+"/" + region,
			url:_regionalForecastURL+region,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseRegionalForecast,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:region,
			waitType:blockType.DATE
		});

		function parseRegionalForecast(data) {
			var regionalForecast;

			function processSections(period){
				 sections = [];
				 $(period).find('Paragraph').each(function(){
					 sections.push( new metoffice.regionalForecastSection({
						title:_parseString(this,'title',true),
						description:_parseString(this)
					}));
				 });
				 return sections;
			}

			$(data).find('RegionalFcst').each(function(){
				var regionId = _parseString(this,'regionId',true);
				var issuedAt= utility.ISO8601toDate(_parseString(this,'issuedAt',true));
				var createdOn= utility.ISO8601toDate(_parseString(this,'createdOn',true));
				var day1To2 = [];
				var day3To5 = [];
				var day6To15 = [];

				$(this).find('Period[id="day1to2"]').each(function(){
					 day1To2 = processSections(this);
				});
				$(this).find('Period[id="day3to5"]').each(function(){
					 day3To5 = processSections(this);
				});
				$(this).find('Period[id="day6to15"]').each(function(){
					 day6To15 = processSections(this);
				});

				regionalForecast= new metoffice.regionalForecast({
					region:utility.getRegionFromId(regionId),
					createdOn:createdOn,
					issuedAt:issuedAt,
					day1To2:day1To2,
					day3To5:day3To5,
					day6To15:day6To15
				});
			});

			return regionalForecast;
		}
	};

	// Time zone
	var _timezone = null;

	timezone.getById = function(options)
	{
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (timezone.getById)";
		}

		var id = options.id;

		_getData({
			cacheKey:_timezoneCacheKey+"/" + id,
			url:_timezoneURL+"Offset/"+id,
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseTimezone,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:id,
			blockType:options.blockType
		});

		function parseTimezone(data) {
			var timezone;
			$(data).find('UtcOffset').each(function(){

				timezone = new metoffice.timezone({
					id:_parseString(this,'timeZone', true),
					offset:_parseNumber(this)
				});
			});

			return timezone;
		}
	};


	timezone.applyTimezoneById = function(options)
	{
		options= (_validateExists(options))?options:{};

		if(!_validatePopulated(options.id))
		{
			throw "No id parameter specified (timezone.applyTimezoneById)";
		}

		timezone.getById(override(options,
			{
				success: function(data){
					metoffice.setTimezone(data);
					if(_validateExists(options.success))
					{
						options.success(data);
					}
				},
				blockType:blockType.DATE
			})
		);
	};

	timezone.getAllTimezoneIds = function(options){
		options= (_validateExists(options))?options:{};

		_getData({
			cacheKey:_timezoneCacheKey+"/" + "AllIds",
			url:_timezoneURL+"Ids",
			dataType: "xml",
			timeToLive : options.timeToLive,
			timeout : options.timeout,
			onDataCallback:parseAllTimezoneIds,
			onSuccessCallback:options.success,
			onErrorCallback:options.error,
			ident:"AllTimezoneIds"
		});

		function parseAllTimezoneIds(data) {
			var ids = [];
			$(data).find('AvailableTimeZone').each(function(){
				ids.push(_parseString(this));

			});

			return ids;
		}
	};

	var ONE_MINUTE_MS = 60000;
	var ONE_HOUR_MS = 3600000;
	var HALF_DAY_MS = 43200000;
	var ONE_DAY_MS = 86400000;
	var	DAY_MS_OFFSETS =[0, ONE_DAY_MS, ONE_DAY_MS*2, ONE_DAY_MS*3, ONE_DAY_MS*4, ONE_DAY_MS*5];

	// UTILITY STATIC

	var _conversionFactors = {

		"mph-kmh": {preadd: 0, multiply: 1.609344, postadd: 0},
		"mph-kts": {preadd: 0, multiply: 0.868976242, postadd: 0},
		"kmh-mph": {preadd: 0, multiply: 0.621371192, postadd: 0},
		"kmh-kts": {preadd: 0, multiply: 0.539956803, postadd: 0},
		"kmh-ms": {preadd: 0, multiply: 0.277777778, postadd: 0},
		"kts-mph": {preadd: 0, multiply: 1.15077945, postadd: 0},
		"kts-kmh": {preadd: 0, multiply: 1.85200, postadd: 0},
		"kts-ms": {preadd: 0, multiply: 0.514444444, postadd: 0},
		"ms-mph": {preadd: 0, multiply: 2.23693629, postadd: 0},
		"ms-kmh": {preadd: 0, multiply: 3.6, postadd: 0},
		"ms-kts": {preadd: 0, multiply: 1.94384449, postadd: 0},
        "mps-kmh": {preadd: 0, multiply: 3.6, postadd: 0},
        "mps-kts": {preadd: 0, multiply: 1.94384449, postadd: 0},
		"hpa-ins": {preadd: 0, multiply: 0.02953337, postadd: 0},
		"ins-hpa": {preadd: 0, multiply: 33.86, postadd: 0},

		"degc-c": {
			preadd: 0,
			multiply: 1.0,
			postadd: 0
		},
		"degc-f": {
			preadd: 0,
			multiply: 1.80,
			postadd: 32
		},
		"c-f": {
			preadd: 0,
			multiply: 1.80,
			postadd: 32
		},
		"f-c": {
			preadd: -32,
			multiply: 0.555555555555,
			postadd: 0
		},
		"mph-km/h": {
			preadd: 0,
			multiply: 1.609344,
			postadd: 0
		},
		"mph-kt": {
			preadd: 0,
			multiply: 0.868976242,
			postadd: 0
		},
		"mph-ms": {
			preadd: 0,
			multiply: 0.44704,
			postadd: 0
		},
		"km/h-mph": {
			preadd: 0,
			multiply: 0.621371192,
			postadd: 0
		},
		"km/h-kt": {
			preadd: 0,
			multiply: 0.539956803,
			postadd: 0
		},
		"km/h-ms": {
			preadd: 0,
			multiply: 0.277777778,
			postadd: 0
		},
		"kt-mph": {
			preadd: 0,
			multiply: 1.15077945,
			postadd: 0
		},
		"kt-km/h": {
			preadd: 0,
			multiply: 1.85200,
			postadd: 0
		},
		"kt-ms": {
			preadd: 0,
			multiply: 0.514444444,
			postadd: 0
		},
		"mps-mph": {
			preadd: 0,
			multiply: 2.23693629,
			postadd: 0
		},
		"mps-km/h": {
			preadd: 0,
			multiply: 3.6,
			postadd: 0
		},
		"mps-kt": {
			preadd: 0,
			multiply: 1.94384449,
			postadd: 0
		},
		"hpa-in": {
			preadd: 0,
			multiply: 0.02953337,
			postadd: 0
		},
		"in-hpa": {
			preadd: 0,
			multiply: 33.86,
			postadd: 0
		},
		"mm-in": {
			preadd: 0,
			multiply: 0.039,
			postadd: 0
		},
		"in-mm": {
			preadd: 0,
			multiply: 25.4,
			postadd: 0
		},
		// metres to m is required to handle xml feeds which contain metres instead of m
		"metres-m": {
			preadd: 0,
			multiply: 1,
			postadd: 0
		},
		"m-metres": {
			preadd: 0,
			multiply: 1,
			postadd: 0
		},
		// percent to % is required to handle xml feeds which contain percent instead of %
		"percent-%": {
			preadd: 0,
			multiply: 1,
			postadd: 0
		}
	};
	
	//STATIC variable for determining how much to round by
	//Maintain lower case
	var _roundingFactor = {
		"metres": 0,
		"m": 0,
		"in": 2,
		"mm": 0,
		"hpa": 0,
		"mph": 0,
		"kt": 0,
		"km/h": 0,
		"c": 1,
		"f": 1,
		"degc": 1,
		"ms": 0,
		"num": 0,
		"deg": 0,
		"code": 0,
		"%": 0,
		"percent": 0,
		"hours": 1
	};

	var utility = {
		convert: function(args) {
			var defaults = {round:true};
			var values = override(defaults, args);
			if (typeof(values.quantity) !== "number" || typeof(values.from) !== "string" || typeof(values.to) !== "string") {
				throw "You must provide an object literal with \"quantity\", \"from\" and \"to\" values";
			}
			values.from = values.from.toLowerCase();
			values.to = values.to.toLowerCase();
			if (values.from == values.to) return values.quantity;
			var key = values.from + "-" + values.to;
			if (typeof(_conversionFactors[key]) == "undefined") throw "That conversion is not supported";

			var interimResult = (((values.quantity + _conversionFactors[key].preadd) * _conversionFactors[key].multiply) + _conversionFactors[key].postadd);

			if (values.round) {
				return Math.round(interimResult);
			} else {
				return interimResult;
			}
		},
		ISO8601toDate: function (string) {
			if((typeof string) =='undefined' || string==null || string.length <=0 ){
				return null;
			}

			var regexp = "([0-9]{4})(-([0-9]{2})(-([0-9]{2})" +
				"(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\\.([0-9]+))?)?" +
				"(Z|(([-+])([0-9]{2}):([0-9]{2})))?)?)?)?";
			var d = string.match(new RegExp(regexp));

			var offset = 0;
			var timezone = 0;
			var date = new Date();



			if(metoffice.getTimezone() == null)
			{
				date.setUTCFullYear(d[1],(d[3]-1),d[5]);
				(d[7])? date.setUTCHours(d[7]):date.setUTCHours(0);
				(d[8])? date.setUTCMinutes(d[8]):date.setUTCMinutes(0);
				(d[10])? date.setUTCSeconds(d[10]):date.setUTCSeconds(0);
				(d[12])? date.setUTCMilliseconds((parseFloat("0." + d[12]) * 1000)):date.setUTCMilliseconds(0);
				if (d[14]) {
					offset = (parseInt(d[16],10) * 60) + parseInt(d[17],10);
					offset *= ((d[15] == '-') ? 1 : -1);
				}
			}
			else{
				date.setFullYear(d[1],(d[3]-1),d[5]);
				(d[7])? date.setHours(d[7]):date.setHours(0);
				(d[8])? date.setMinutes(d[8]):date.setMinutes(0);
				(d[10])? date.setSeconds(d[10]):date.setSeconds(0);
				(d[12])? date.setMilliseconds((parseFloat("0." + d[12]) * 1000)):date.setMilliseconds(0);
				if (d[14]) {
					offset = (parseInt(d[16],10) * 60) + parseInt(d[17],10);
					offset *= ((d[15] == '-') ? 1 : -1);
					offset +=(metoffice.getTimezone().getOffset()*60);
				}
				else if(d[7] && d[7]!==null && d[7].length > 0)
				{
					offset = metoffice.getTimezone().getOffset()*60;
				}
			}

//			offset -= date.getTimezoneOffset();
			date.setTime((date.getTime() + ((offset) * 60 * 1000)));
			return date;
		},
		ONE_MINUTE_MS : ONE_MINUTE_MS,
		ONE_HOUR_MS : ONE_HOUR_MS,
		HALF_DAY_MS : HALF_DAY_MS,
		ONE_DAY_MS : ONE_DAY_MS,
		DAY_MS_OFFSETS: DAY_MS_OFFSETS,
		setTimeToMidnight : function(inDate){
			if(inDate && inDate.getTime)
			{
				var newDate = new Date(inDate.getTime());
				newDate.setHours(0, 0, 0, 0);
				return newDate;
			}
			return null;
		},
		dateFormat: function (date) {
			var month_name;
			month_name = ["January","February","March","April","May","June","July","August","September","October","November","December"];
			month_name = ["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];
			var returnValue = (date.getDate() < 10 ? "0" : "") + date.getDate() + " " + month_name[date.getMonth()] + " " + date.getFullYear() + ", " + (date.getHours() < 10 ? "0" : "") + date.getHours() + ":" + (date.getMinutes() < 9 ? "0" : "") + date.getMinutes();
			return returnValue;
		},
		arrayContains: function(target, element) {
			var alen = target.length;
			for (var i = 0; i < alen; i++) {
				if (target[i] === element) return true;
			}
			return false;
		},
		isDaylightSaving: function(options) {
			options = (_validateExists(options))?options:{};
			if(!_validatePopulated(options.success))
			{
				throw "No success parameter specified (utility.isDaylightSaving)";
			}
			$.ajax({
				url: _daylightSavingURL,
				method: "GET",
				timeout: defaultTimeout,
				cache: false,
				success: function(data) {
					try {
						var hourOffset = parseInt(data, 10);
						if (isNaN(hourOffset)) {
							options.error("Error fetching daylight saving: Retreived data is not a number");
						} else {
							options.success({daylightSaving: (hourOffset != 0), offset: hourOffset});
						}
					}
					catch(err) {
						options.error("Error fetching daylight saving: "+err);
					}
				},
				error: function() {
					options.error("Error fetching daylight saving: ");
				}
			});
		},
		getDisplayPeriod: function(options) {
			options = (_validateExists(options))?options:{};
			if (!_validatePopulated(options.success)) {
				throw "No success parameter specified (utility.getDisplayPeriod)";
			}
			$.ajax({
				url: _displayPeriodURL,
				method: "GET",
				timeout: defaultTimeout,
				cache: false,
				success: function(data) {
					try {
						data = data.toLowerCase().substr(0,3);
						if (data == "day") {
							options.success(displayPeriod.DAY);
						} else {
							options.success(displayPeriod.NIGHT);
						}
					}
					catch(err) {
						options.success(displayPeriod.DAY);
					}
				},
				error: function() {
					options.success(displayPeriod.DAY);
				}
			});
		},
		isAPostcode: function(value) {
			//TODO - create a function that better validates a postcode so that if a full postcode is entered, it returns the correct first half outward block.
			value = value.replace(/ /ig,'');
			//REGULAR EXPRESSIONS - If Postcode
			//TO DO - Make sure regular expression works for all postcodes that we're allowing
			if(value.match(/[a-zA-z]{1,2}[0-9]{1,2}[a-zA-z]{0,1} [0-9][a-zA-Z]{2}/) || value.match(/[a-zA-z]{1,2}[0-9]{1,2}/)){
				return true;
			} else {
				return false;
			}
		},
		convertVisibiltyToWords : function(vis) {
			if (vis !== null && vis >= 0) {
				if (vis < 1000) {
					return 'Very poor';
				}
				if (vis >= 1000 && vis < 4000) {
					return 'Poor';
				}
				if (vis >= 4000 && vis < 10000) {
					return 'Moderate';
				}
				if (vis >= 10000 && vis < 20000) {
					return 'Good';
				}
				if (vis >= 20000 && vis < 40000) {
					return 'Very good';
				}
				if (vis >= 40000) {
					return 'Excellent';
				}
			}
			return 'N/A';
		},

		getRegionFromId : function(id){
            id = id ? id.toLowerCase() : null;
			if(id && _regions[id] ){
				return _regions[id];
			}
			else{
				return null;
			}
		}
	};

	// WARNING STATICS

	// warningLevel getMaxWarningLevel(string region)
	// gets the highest warning level across all warnings
	warning.getMaxWarningLevel = function(options) {
		options= (_validateExists(options))?options:{};

		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				var result = metoffice.warningLevel.GREEN;
				for (var i = 0; i < data.length; i++) {
					var nextValue = data[i].getLevel();
					if (nextValue.getValue() > result.getValue()) {
						result = nextValue;
					}
				}
				options.success(result);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	// warningGetBatchId
	// gets the batch ID for the latest set of warnings
	warning.getBatchId = function(options) {
		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				options.success(data.batchId);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	// warningGetServerTime
	// gets the server time for the latest set of warnings
	warning.getServerTime = function(options) {
		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				options.success(data.serverTime);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	// warningLevel getMaxWarningLevelByRegion(string region)
	// gets the highest warning level for a region across all warnings
	warning.getMaxWarningLevelByRegion = function(options) {
		options= (_validateExists(options))?options:{};

		if ( ! _validatePopulated(options.region) ) {
			throw "No region parameter specified (getMaxWarningLevelByRegion)";
		}

		warning.getWarningsByRegion({
            timeToLive:options.timeToLive,
			region: options.region,
			success: function(data) {
				var result = null;
				for (var i = 0; i < data.length; i++) {
					var nextValue = data[i].getMaxLevelByRegion(options.region);
					if (nextValue.getValue() > result) {
						result = nextValue;
					}
				}
				options.success(result);
			},
			error: function(e) {
				options.error(e);
			}
		});

	};

	// warning[] getAllWarnings()
	// gets all warnings
	warning.getAllWarnings = function(options) {
		options= (_validateExists(options))?options:{};

		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				options.success(data);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	// warning[] getWarningsByRegion(string region)
	// gets all warnings in a region
	warning.getWarningsByRegion = function(options) {
		options= (_validateExists(options))?options:{};

		if (! _validatePopulated(options.region) ) {
			throw "No region parameter specified (getWarningsByRegion)";
		}
		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				var warningSet = [];
				for (var i = 0; i < data.length; i++) {
					if (data[i].includesRegion(options.region)) {
						warningSet.push(data[i]);
					}
				}
				options.success(warningSet);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	// warningLevel getMaxWarningLevelByType(warningWeatherType weatherType)
	// gets the max warning level (red, etc.) by weather type: warning level (national)
	warning.getMaxWarningLevelByType = function(options) {
		options= (_validateExists(options))?options:{};

		if (! _validatePopulated(options.weatherType)) {
			throw "No weatherType parameter specifiedParameter (getMaxWarningLevelByType)";
		}
		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				var result = _determineMaxWarningByType(data, options.weatherType);
				options.success(result ? result.getLevel() : null);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	function _determineMaxWarningByType(data, weatherType) {
		var result = null;
		for (var i = 0; i < data.length; i++) {
            var nextValue = data[i];
			if (nextValue.getWeatherType() == weatherType) {

				if (result === null
                        // a higher level
                    || nextValue.getLevel().getValue() > result.getLevel().getValue()
                        // the same level but the kind is higher (alert is higher than warning)
                    || (nextValue.getLevel().getValue() === result.getLevel().getValue()
                        && nextValue.getKind().getValue() > result.getKind().getValue())) {
					result = nextValue;
                }
			}
		}
		return result;
	}

	// {type: warningWeatherType, level: warningLevel}[] getMaxWarningLevelByAllTypes()
	// gets warning level for each of the five types.
	warning.getMaxWarningLevelByAllTypes = function(options) {
		options= (_validateExists(options))?options:{};

		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				var results = [];
				for (var i = 0; i < warningWeatherType.items.length; i++) {
                    var w = _determineMaxWarningByType(data, warningWeatherType.items[i]);
					var individualResult = {
                        type: warningWeatherType.items[i],
                        level: w ? w.getLevel() : null,
                        kind: w ? w.getKind() : null
                    };
					results.push(individualResult);
				}
				options.success(results);
			},
			error: function(e) {
				options.error(e);
			}
		});

	};


	var _readCachedWarnings = function(data, options){
		options= (_validateExists(options))?options:{};
		var regionId = (_validatePopulated(options.region))?options.region:null;

		var _cachedWarnings = [];	// raw warnings from the Cache

		for (var i = 0; i < data.length; i++) {
			var _thisCachedWarning = {};
			_thisCachedWarning= data[i];
			for (var j = 0; j < _thisCachedWarning.getZones().length; j++) {
				var _thisCachedZone = _thisCachedWarning.getZones()[j];
				for (var k = 0; k < _thisCachedZone.getWarningRegions().length; k++) {
					var _thisCachedWarningRegion = _thisCachedZone.getWarningRegions()[k];
					if(regionId === null || regionId === _thisCachedWarningRegion.getRegion().getId() ){
						var _cachedWarning = { region: _thisCachedWarningRegion.getRegion(),
												warningText: _thisCachedWarning.getWarningText(),
												forecasterText:_thisCachedWarning.getForecasterText(),
												type: _thisCachedWarning.getWeatherType(),
												modifiedDate: _thisCachedWarning.getModifiedDate(),
												createdDate: _thisCachedWarning.getCreatedDate(),
												validFrom: _thisCachedWarning.getValidFrom(),
												validTo: _thisCachedWarning.getValidTo(),
												status: _thisCachedWarning.getStatus(),
												level: _thisCachedZone.getLevel(),
												kind: _thisCachedWarning.getKind(),
												impact: _thisCachedWarning.getImpact(),
												likelihood: _thisCachedWarning.getLikelihood(),
												safetyTypes: _thisCachedWarning.getSafetyTypes(),
												safetyText: _thisCachedWarning.getSafetyText(),
												iconPosition: _thisCachedWarning.getIconPosition(),
												version: _thisCachedWarning.getVersion()
											};

						_cachedWarnings.push(_cachedWarning);
					}
				}
			}
		}
		return _cachedWarnings;
	};

	// gets most severe level & kind, an array of warnings for each weather type with highest level & kind
	// and then within these warning types, an array of regional warnings with region code & name, warning level & kind.
	//Object designed to replicate the xml returned from
	//	mostSevereWarning	.highest								warningLevel		Green, Yellow, Amber, Red	(val, display)
	//						.kind									warningKind			ALERT, WARNING				(val, display)
	//						.warningsByWeatherType
	//									.type						weatherType			SNOW, RAIN, ICE, FOG, WIND	(val, display)
	//									.highest					warningLevel		Green, Yellow, Amber, Red
	//									.kind						warningKind			ALERT, WARNING
	//									.regionalWarnings	.code
	//														.name
	//														.level	warningLevel		Green, Yellow, Amber, Red
	//														.kind	warningKind			ALERT, WARNING
	warning.getMostSevereWarningAll = function(options) {
		options= (_validateExists(options))?options:{};

		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				parseMostSevereWarningAll(data,options);
			},
			error: function(e) {
				options.error(e);
			}
		});

	};

	warning.getMostSevereWarningForRegion = function(options) {
		options= (_validateExists(options))?options:{};
		if (! _validatePopulated(options.region)) {
			throw "No region parameter specified (getMostSevereWarningForRegion)";
		}
		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				parseMostSevereWarningAll(data,options);
			},
			error: function(e) {
				options.error(e);
			}
		});

	};

	function parseMostSevereWarningAll(data, options){
		var results = [];
		var _cachedWarnings = _readCachedWarnings(data, options);

		// Now have an array full of _cachedWarnings, sort them into weather types and pick the highest
		var mostSevereWarning = new metoffice.mostSevereWarning({ highest:warningLevel.GREEN,  kind:warningKind.ALERT, warningsByType : {}, warningsByDate:[]});

		for ( var i = 0; i < _cachedWarnings.length; i++ ) {
			var _thisCachedWarning = _cachedWarnings[i];
			// set the highest and kind in MostSeverWarning
			if ( _thisCachedWarning.level.getValue() > mostSevereWarning.getHighest().getValue() ) {
				mostSevereWarning._values.highest = _thisCachedWarning.level;
			}
			if ( _thisCachedWarning.kind.getValue() > mostSevereWarning.getKind().getValue() ) {
				mostSevereWarning._values.kind = _thisCachedWarning.kind;
			}

			// Prepare the regionalWarning for addition
			var _thisRegionalWarning = createRegionalWarning(_thisCachedWarning);

			// check if there is a warningByWeatherType for this type
			var _thisWarningByWeatherType = mostSevereWarning.getWarningsByType()[_thisCachedWarning.type];
			if( typeof _thisWarningByWeatherType == "undefined"){
				_thisWarningByWeatherType = new metoffice.mostSevereWarningType({ type: _thisCachedWarning.type,
					highest: _thisCachedWarning.level,
					kind: _thisCachedWarning.kind,
					regionalWarnings:[]});
				mostSevereWarning._values.warningsByType[_thisCachedWarning.type]=_thisWarningByWeatherType;
			}
			else{
				if ( _thisCachedWarning.level.getValue() > _thisWarningByWeatherType.getHighest().getValue() ) {
					_thisWarningByWeatherType._values.highest = _thisCachedWarning.level;
				}
				if ( _thisCachedWarning.kind.getValue() > _thisWarningByWeatherType.getKind().getValue() ) {
					_thisWarningByWeatherType._values.kind = _thisCachedWarning.kind;
				}
			}

			_thisWarningByWeatherType._values.regionalWarnings.push(_thisRegionalWarning);
		}

		options.success(mostSevereWarning);
	}

	// Similar function to above but splits warnings into five days (starting today) then by weather type and with regional warnings
	// Regional warnings will be repeated if start or end (or middle!) falls within the specified day
	//
	// gets most severe level & kind, an array of warnings for each weather type with highest level & kind
	// and then within these warning types, an array of regional warnings with region code & name, warning level & kind.
	//	mostSevereWarning	.highest										Green, Yellow, Amber, Red
	//						.highestVal										0-3
	//						.kind											ALERT, WARNING
	//						.kindVal										0,1
	//						.warningsByDate[]
	//								.day									0-4
	//								.date									String date
	//								.highest								Green, Yellow, Amber, Red
	//								.highestVal								0-4
	//								.kind									ALERT, WARNING
	//								.kindVal								0,1
	//								.warningsByWeatherType[]
	//										.type							SNOW, RAIN, ICE, FOG, WIND
	//										.typeVal						0-4
	//										.highest						Green, Yellow, Amber, Red
	//										.highestVal						0-4
	//										.kind							ALERT, WARNING
	//										.kindVal						0,1
	//										.regionalWarnings
	//												.code					2 char region code
	//												.region					Text description
	//												.level					Green, Yellow, Amber, Red
	//												.kind					ALERT, WARNING
	//												.validFrom				Date
	//												.validTo				Date
	//												.type					SNOW, RAIN, ICE, FOG, WIND
	warning.getMostSevereWarningAllByDate = function(options) {
		options= (_validateExists(options))?options:{};

		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				parseMostSevereWarningsByDate(data, options);
			},
			error: function(e) {
				options.error(e);
			}
		});

	};

	warning.getMostSevereWarningForRegionByDate = function(options) {
		options= (_validateExists(options))?options:{};
		if (! _validatePopulated(options.region)) {
			throw "No region parameter specified (getMostSevereWarningForRegionByDate)";
		}

		_retrieveWarnings({
            timeToLive:options.timeToLive,
			success: function(data) {
				parseMostSevereWarningsByDate(data, options);
			},
			error: function(e) {
				options.error(e);
			}
		});
	};

	function parseMostSevereWarningsByDate(data, options){
		var todayNum = utility.setTimeToMidnight((data.serverTime)?data.serverTime:new Date()).getTime();

		var cachedWarnings = _readCachedWarnings(data, options);

		var mostSevereWarning = new metoffice.mostSevereWarning({ highest:warningLevel.GREEN,  kind:warningKind.ALERT, warningsByType:{},warningsByDate:[] });

		// Now have an array full of cachedWarnings, check dates and create nodes where appropriate
		for ( var i = 0; i < cachedWarnings.length; i++ ) {
			var thisCachedWarning = cachedWarnings[i];

			// create the date nodes for the warnings if not found
			for (var iDay = 0; iDay < 5; iDay++) {

				if((thisCachedWarning.validFrom.getTime() < (todayNum + utility.DAY_MS_OFFSETS[iDay+1])) && (thisCachedWarning.validTo.getTime() >= (todayNum + utility.DAY_MS_OFFSETS[iDay]))){
					// date is in valid range, so check and set the highest and kind in MostSeverWarning
					if ( thisCachedWarning.level.getValue() > mostSevereWarning.getHighest().getValue() ) {
						mostSevereWarning._values.highest = thisCachedWarning.level;
						mostSevereWarning._values.kind = thisCachedWarning.kind;
					} else if ( thisCachedWarning.level.getValue() == mostSevereWarning.getHighest().getValue() &&
								thisCachedWarning.kind.getValue() > mostSevereWarning.getKind().getValue()) {
						mostSevereWarning._values.kind = thisCachedWarning.kind;
					}

					if (!warningByDateFound(mostSevereWarning.getWarningsByDate(), iDay)) {
						 mostSevereWarning._values.warningsByDate.push(createWarningByDate(thisCachedWarning, iDay));
					}

					// now there's definitely a wBD, iterate over them and look for warningByWeatherType
					$.each(mostSevereWarning.getWarningsByDate(), function() {
						var thisWBD = this;
						if ( thisCachedWarning.level.getValue() > thisWBD.getHighest().getValue() ) {
							thisWBD._values.highest = thisCachedWarning.level;
							thisWBD._values.kind = thisCachedWarning.kind;
						} else if ( thisCachedWarning.level.getValue() == thisWBD.getHighest().getValue() &&
									thisCachedWarning.kind.getValue() > thisWBD.getKind().getValue()) {
							thisWBD._values.kind = thisCachedWarning.kind;
						}

						if (thisWBD.getDay() === iDay) {
							if (!warningByWeatherTypeFound(thisWBD.getWarningsByType(), thisCachedWarning.type)) {
								thisWBD._values.warningsByType[thisCachedWarning.type]=createWarningByWeatherType(thisCachedWarning);
							}
							// now there's definitely a wBWT, find it and add regional warning
							$.each(thisWBD.getWarningsByType(), function() {
								var thisWBWT = this;
								if ( thisCachedWarning.level.getValue() > thisWBWT.getHighest().getValue() ) {
									thisWBWT._values.highest = thisCachedWarning.level;
									thisWBWT._values.kind = thisCachedWarning.kind;
								} else if ( thisCachedWarning.level.getValue() == thisWBWT.getHighest().getValue() &&
											thisCachedWarning.kind.getValue() > thisWBWT.getKind().getValue()) {
									thisWBWT._values.kind = thisCachedWarning.kind;
								}
								if(thisWBWT.getType().getValue() === thisCachedWarning.type.getValue())
								{
									var regWarning = regionalWarningFound(thisWBWT.getRegionalWarnings(), thisCachedWarning.region);
									if(!_validateExists(regWarning)){
										regWarning = createRegionalWarning(thisCachedWarning);
										thisWBWT._values.regionalWarnings.push(regWarning);
									}
									else if(thisCachedWarning.level.getValue()>regWarning.getLevel().getValue()){
										regWarning._values.level = thisCachedWarning.level;
									}
								}
							});
						}
					});
				}
			}
		}
		options.success(mostSevereWarning);
	}

	// Functions to create nodes for mostSevereWarning record
	// inWarning will be a cachedWarning containing:  code, name, level, levelVal, kind, kindVal, type, typeVal, validFrom & validTo
	// which should be enough to make up the nodes and elements
	function createWarningByDate(inWarning, inDay) {

		var thisDate = new Date();
		thisDate.setDate(thisDate.getDate() + inDay); // today + inDay
		thisWarningByDate = new metoffice.mostSevereWarningDate({ day: inDay, date: thisDate,
								highest: inWarning.level,
								kind: inWarning.kind,
								warningsByType:{}});
		return thisWarningByDate;
	}

	function createWarningByWeatherType(inWarning) {
		var thisWarningByWeatherType = new metoffice.mostSevereWarningType({ type: inWarning.type,
								highest: inWarning.level,
								kind: inWarning.kind,
								regionalWarnings:[] });
		return thisWarningByWeatherType;
	}

	function createRegionalWarning(inWarning) {

		var thisRegionalWarning = new metoffice.mostSevereRegionalWarning({ region: inWarning.region,
			 					warningText: inWarning.warningText,forecasterText:inWarning.forecasterText,
								createdDate: inWarning.createdDate, modifiedDate: inWarning.modifiedDate,
								validFrom: inWarning.validFrom, validTo: inWarning.validTo,
								status: inWarning.status, level: inWarning.level, kind: inWarning.kind,
								impact: inWarning.impact, likelihood: inWarning.likelihood, safetyText: inWarning.safetyText,
								safetyTypes: inWarning.safetyTypes, iconPosition: inWarning.iconPosition,
								version: inWarning.version, type: inWarning.type});
		return thisRegionalWarning;
	}

	// Functions returning boolean indicating if mostSeverWarning contains a specific node
	function warningByDateFound(inWarningsByDate, inDay) {
		var found = false;
		if ( inWarningsByDate.length ) {
			for (var i=0; i<inWarningsByDate.length; i++) {
				if (inWarningsByDate[i].getDay() === inDay){
					return true;
				}
			}
		}
		return false;
	}

	function warningByWeatherTypeFound(inWarningsByWeatherType, inTypeVal) {
		return _validateExists(inWarningsByWeatherType[inTypeVal]);
	}

	function regionalWarningFound(inRegionalWarnings, region) {
		if ( inRegionalWarnings.length ) {
			for (var i = 0; i < inRegionalWarnings.length; i++) {
				if (inRegionalWarnings[i].getRegion().getId() == region.getId()){
					return inRegionalWarnings[i];
				}
			}
		}
		return null;
	}

	// gets warnings XML, parse into array of warnings, store in cache.
	function _retrieveWarnings(options) {
		_getData({
			cacheKey: _warningsCacheKey+"/" +"All",
			url: _warningsURL,
			dataType: "xml",
			onDataCallback: parseWarnings,
			timeout : options.timeout,
			timeToLive : (_validatePopulated(options.timeToLive))?options.timeToLive:-1,
			onSuccessCallback: options.success,
			onErrorCallback: function(error) {
				options.error("Warnings could not be retrieved: "+error);
			},
			waitType:blockType.DATE
		});

		function parseWarnings(xml) {
			var warnings = [];
			var jWarnings = $(xml).find("NSWWarning");

			var serverTime = utility.ISO8601toDate($(xml).find('WarningList').find("[nodeName='gml:description']").text());
			var warningsBatchID= $(xml).find('WarningList').attr('gml:id');

			jWarnings.each(function() {
				var jWarning = $(this);
				var warningText = _parseString(jWarning,'warningText',false);
				var forecasterText = _parseString(jWarning,'forecasterText',false);
				var weather = warningWeatherType.parse(_parseString(jWarning,'weather',false));
				var createdDate = utility.ISO8601toDate(_parseString(jWarning,'createdDate',false));
				var modifiedDate = utility.ISO8601toDate(_parseString(jWarning,'modifiedDate',false));
				var validFrom = utility.ISO8601toDate(_parseString(jWarning,'validFrom',false));
				var validTo = utility.ISO8601toDate(_parseString(jWarning,'validTo',false));
				var status = warningStatus.parse(_parseString(jWarning,'status',false));
				var overallWarningLevel = warningLevel.parse(_parseString(jWarning,'> warningLevel',false));
				var kind = warningKind.parse(_parseString(jWarning,'warningClass',false));
				var likelihood = _parseNumber(jWarning, 'warningLikelihood',false);
				var impact = _parseNumber(jWarning, 'warningImpact',false);
				var safetyListStr = _parseString(jWarning,'safetyList',false);
				var safetyTypeList = [];
				if(safetyListStr !== null && safetyListStr.length > 0){
					var elements = safetyListStr.split(" ");
					for(var k = 0 ; k < elements.length;k++){
						var type = safetyType.parse(elements[k]);
						if(type !== null){
							safetyTypeList.push(type);
						}
					}
				}


				var safetyText = _parseString(jWarning,'safetyText',false);
				var warningId = _parseString(jWarning,'warningId',false);
				var version = _parseNumber(jWarning,'version',false);
				var iconPos = null;
				var iconPosStr = _parseString(jWarning,'iconPos',false);
				if(iconPosStr!==null){
					var list = iconPosStr.split(",");
					if(list !== null && list.length === 2){
						iconPos = new point({lat:new Number(list[0]).valueOf(),lon:new Number(list[1]).valueOf()});
					}
				}
				var zonesList = [];

				var jZones = jWarning.find("Zone");
				jZones.each(function() {
					var jZone = $(this);

					var zoneWarningLevel = warningLevel.parse(_parseString(jZone,'warningLevel',false));
					var likelihood = _parseNumber(jZone,'likelihood',false);
					var impact = _parseNumber(jZone,'impact',false);
					var zoneId = _parseString(jZone,'zoneId',false);

					var polygon = [];
					var coordString = jZone.find("[nodeName='gml:coordinates']").text().replace("\n", " ").split(" ");
					$(coordString).each(function() {
						var coords = this.split(",");
						if(coords.length==2){
							polygon.push(new point({
								lat: parseFloat(coords[0]),
								lon: parseFloat(coords[1])
							}));
						}
					});

					var zoneRegions = [];
					var jRegions = jZone.find("Region");

					jRegions.each(function() {
						var jRegion = $(this);
						var region = utility.getRegionFromId(_parseString(jRegion,'code',false));
						var subRegions = [];

						jRegion.find("areaName").each(function(){
							subRegions.push(_parseString(this));
						});

						var myRegion = new warningRegion({region: region, subRegions: subRegions});
						zoneRegions.push(myRegion);
					});

					zonesList.push(new zone({id: zoneId, polygon: polygon, warningRegions: zoneRegions, level: zoneWarningLevel, likelihood: likelihood, impact: impact}));
				});
				var thisWarning = new warning({
					id: warningId, warningText: warningText, forecasterText: forecasterText, weatherType: weather, createdDate: createdDate, modifiedDate: modifiedDate, validFrom: validFrom,
					validTo: validTo, status: status, level: overallWarningLevel, kind: kind, likelihood: likelihood, impact: impact, safetyTypes: safetyTypeList, safetyText: safetyText, iconPosition: iconPos, version: version,
					zones: zonesList
				});
				warnings.push(thisWarning);
			});
			warnings.serverTime = serverTime;
			warnings.batchId = warningsBatchID;
			return warnings;
		}
	}

	//PUBLIC FUNCTIONS
	var log = function() {
		_doLog(true, Array.prototype.slice.call(arguments,0));
	};
	var _cacheTypeQueue = new optionsQueue({ident:"cacheTypeQueue"});

	//PRIVATE OBJECTS
	var cache = {
		clear: function() {
			_cache = {};
			_cacheTypeQueue.clear();
		},
		get: function(key) {
			var data = _cache[key];
			if(typeof data !== "undefined" && data !== null )
			{
				var currentTime = new Date().getTime();
				if(data.time === null || data.time <= 0 || currentTime < data.time )
				{
					return _cache[key].data;
				}
				else{
					this.remove(key);
				}
			}
			return null;
		},
		put: function(key, data, options) {
			var time = 0;
			if(options)
			{
				if(options.timeToLive && options.timeToLive > 0)
				{
					time = new Date().getTime()+options.timeToLive;
				}
				if(options.waitType && options.waitType != null){
					_cacheTypeQueue.push(options.waitType.getValue(),key);
				}
			}

			_cache[key] = {data:data,time:time};
		},
		remove: function(key){
			_cache[key] = null;
		},
		clearType: function(type){
			var keyArray = _cacheTypeQueue.pop(type.getValue());
			if(keyArray && keyArray != null)
			{
				for(var i = 0 ; i < keyArray.length; i++){
					this.remove(keyArray[i]);
				}
			}
		}
	};

	function optionsQueue(queueOptions){
		this._optionsQueue = {};
        this.push = function(key, options){
            var isNew = false,
                keyArray = this._optionsQueue[key];
            if(typeof(keyArray) == "undefined" || keyArray==null)
            {
                keyArray = [];
                isNew = true;
            }
            keyArray.push(options);
            this._optionsQueue[key]=keyArray;
            return isNew;
        };

        /*
         * Returns the array of options or null
         * */
        this.pop = function(key){
            var keyArray = this._optionsQueue[key];
            this._optionsQueue[key] = null;
            return (keyArray)?keyArray:[];
        };

        this.popObject = function(key, object){
        	var keyArray = this._optionsQueue[key];
            if(_validateExists(keyArray) && keyArray.length > 0)
            {
            	for(var i = 0 ; i < keyArray.length; i++){
            		if(keyArray[i] == object){
            			if(keyArray.length==1){
            				this._optionsQueue[key] = null;
            			}
            			return keyArray.splice(i,1);
            		}
            	}
            }
            return null;
        };

        this.contains = function(key){
        	var keyArray = this._optionsQueue[key];
        	if(_validateExists(keyArray) && keyArray.length > 0)
            {
            	return true;
            }
            return false;
        };

        this.containsObject = function(key, object){
        	var keyArray = this._optionsQueue[key];
        	if(_validateExists(keyArray) && keyArray.length > 0)
            {
            	for(var i = 0 ; i < keyArray.length; i++){
            		if(keyArray[i] == object){
            			return true;
            		}
            	}
            }
            return false;
        };

        this.clear = function(){
        	this._optionsQueue = {};
        };
    }

	var dataQueue = new optionsQueue({ident:"dataQueue"});

	var requestQueue = new optionsQueue({ident:"requestQueue"});

	var blockQueue = new optionsQueue({ident:"blockQueue"});

	var waitQueue = new optionsQueue({ident:"waitQueue"});


	//PRIVATE FUNCTIONS
	// private functions in library aren't assigned into the returned "library" object.
	function buildEntity(template, methods){
        // internal functions first
        function accessorName(name){
            return "get" + name.slice(0,1).toUpperCase() + name.slice(1,name.length);
        }

        function simpleAccessor(name,defaultValue){
            return function(){
                var val = this._values[name];
                return (val !== null) ? val : defaultValue;
            };
        }

        function functionAccessor(name, fn){
            return function(){
                var val = this._values[name];
                return (val !== null) ? new fn(val) : null;
            };
        }

		var constructor = function( values ){
			this._values = values;
		};
		var proto = constructor.prototype = {
            values:function(){return this._values;}
        };

		for( var name in template ){
			if( template.hasOwnProperty(name)){
                proto[accessorName(name)] = typeof template[name] === 'function' ?
                        functionAccessor(name, template[name]):
                        		simpleAccessor(name,template[name]);
			}
		}

		if( methods !== null ){
			for( var name in methods){
				if( methods.hasOwnProperty(name)){
					proto[name] = methods[name];
				}
			}
		}
		return constructor;
	}

	var _doLog = function(isAPI, toLog) {
		var args = isAPI? "API: ": "",
            air = window.air;
		if(toLog.length > 0) {
			for (var a=0; a < toLog.length; a++) {
				args = args + toLog[a] + " ";
			}
		} else {
			args = "(nothing to log)";
		}
        if( air && air.Introspector && air.Introspector.Console ){
            air.Introspector.Console.log(args);
        }else if( air && air.trace){
            air.trace(args);
        }else if( typeof console !== 'undefined' && console != null) {
			console.log(args);
		}
        // else nothing
	};

	var _externalLog = function() {
		_doLog(false, arguments);
	};

	/**
	 * Requests a set of data. If the data is stored in the cache against the specified cacheKey then that data is returned
	 * otherwise the data is requested via ajax from the specified URL and is passed to the onDataCallback.
	 * onErrorCallback is called if the request fails and successCallback is called if the request
	 * succeeds.
	 *
	 * This implements a request queue to stop multiple identical parallel requests to the same URL from
	 * making parallel identical server requests. Before a server request is made an entry is placed in the requestQueue
	 * to indicate that an asynchronous query agaisnt the url is currently in progress. This entry consists of the
	 * url unique key and the original options passed to this method. An ajax server request is then initiated. In the
	 * meantime if any identical data requests are made then no ajax request is made. Instead the options are added to the queue for the
	 * url key and teh method exits. Once the original server request completes then the data is processed and cached,
	 * then the queue for the url key is retrieved and this method is then re-executed with the options for each queued
	 * request (including the first request that initiated the ajax call). Since the requested data has now been cached
	 * all the original data requests will now be able to extract hte required data from the cache and thier onSuccessCallback
	 * functions will be called in the original request order. This results in one server request being made for all identical requests.
	 *
	 * If no cacheKey is specified then the cache will not be used. This can be used when it is required that a specific data request
	 * is never cached and all requests for that data should retrieve data directly from the server (eg Weather Warnings). In this
	 * situation the requestQueue will still be used to handle parallel requests but later data requests will result in new server request
	 *
	 * If no url is specified then no server request processing will be peroformed. If the data is in the cache then
	 * the successCallback is executed. If not the errorCallback is executed.
	 *
	 * If dataKey is specified then the result of any server requests will be cached against this key rather than the cache key. This
	 * is used when you want to query the cache for a specific piece of data and if it doesn't exist then you wnat to do
	 * a server request against a larger generic data set that contains that data plus other similar pieces of data. Eg extremes are
	 * feteched as a single server request containing the extremes for all regions but are used and cached on a regional basis. In
	 * this situation you specify the cacheKey to be the region you want to check agaisnt hte cache for. If it doesn't exist then a server request
	 * is made to fetch all the extreme data for all regions and caches that against the dataKey. The onDataCallback can then extracts
	 * and cache each individual regional extreme data set agaisnt its own cache key. Subsequent calls to getData with the original regional
	 * cacheKey will then extract the now cached regionalExtremes.
	 *
	 * If no onDataCallback is specified then the data returned from the server will be cached as is.
	 *
	 * options{
	 *  string cacheKey <optional>, - key used to ftech and old data in cache
	 *  string dataKey <optional>, - see above
	 *  string requestKey <optional>, - If specifed the unparsed data returned from the url request will be stored against this key
	 *  string url <optional>, - sever request url
	 *  function Object onDataCallback(data) <optional>, - called to parses data returned from server requests
	 *  function onSuccessCallback(data), - called with fetched data
	 *  function onErrorCallback <optional>, - called if no data could be found or if server errors
	 *  int timeout <optional|default:30000>, - server request timeout
	 *  String type <optional|default:"GET">, - server request type (Get, Put etc)
	 *  String dataType <optional|default:null>, - server reposnse data type (xml, text, json etc)
	 *  number timeToLive <optional:default:null>, - The maximum cache time for elements. Once expires new call indicate that elements are recached. 0 = Never die. -1=never cache
	 *  blockType blockType <optional:default:null>, - If specified this will cause response processing of any outstanding and subsequent requests with a matching waitType to block until this request has
	 *  	completed (ie onSuccessCallback has completed). This process handles request chaining situations where the parsing of a response must occur after a spearte async request has completed.
	 *  	The principle example of this is with  dates in forecasts etc that should only be written after the correct timezone/offset information is retrieved
	 *  blockType waitType <optional:default:null> - If specified the parsing of the response of the request will be blocked whilst any requests with a matching blockType are outstanding. The underlying http request will
	 *  	still be made in parallel but the onDataCallback will not be called until the block is removed.
	 *  }
	 */
	_getData = function(options) {
		options.timeToLive=(_validatePopulated(options.timeToLive))?options.timeToLive:0;
		options.timeout=(_validatePopulated(options.timeout))?options.timeout:defaultTimeout;
		options.blockType=(_validatePopulated(options.blockType))?options.blockType:null;
		options.waitType=(_validatePopulated(options.waitType))?options.waitType:null;
		block(options);
		var useCache = _validatePopulated(options.cacheKey) && options.timeToLive >= 0;
		var useRequestCache = useCache && _allowDataCaching && _validatePopulated(options.requestKey);
		var cacheData = (useCache)?cache.get(options.cacheKey):null;

		if (useCache && cacheData !== null ) {
			if(_validateExists(options.onSuccessCallback))
			{
				options.onSuccessCallback(cacheData);
				unblock(options);
			}
		} else if(options.url){
			if(!_validatePopulated(options.dataKey))
			{
				options.dataKey = options.cacheKey;
			}
			if(dataQueue.push(options.dataKey, options))
			{
				var requestCacheData = (useRequestCache)?cache.get(options.requestKey):null;
				if(useRequestCache && requestCacheData !== null)
				{
					processDataSuccess(useCache, options, requestCacheData);
				}
				else{
					if(!useRequestCache || requestQueue.push(options.requestKey, options))
					{
						try{
							$.ajax({
								type: (options.type)?options.type:"GET",
								url: _buildURL(options.url),
								dataType: options.dataType,
								timeout: options.timeout,
								error: function(XMLHttpRequest, textStatus, errorThrown){
									processError("Status: "+textStatus+" Error: "+errorThrown, options);
								},
								success: function(data,textStatus,XMLHttpRequest) {
									if(useRequestCache)
									{
										cache.put(options.requestKey, data, options);
										$.each(requestQueue.pop(options.requestKey), function(){
											processDataSuccess(useCache, this, data);
										});
									}
									else{
										processDataSuccess(useCache, options, data);
									}
								}
							});
						}
						catch(error){
							processError(error, options);
						}
					}
				}
			}
		}
		else{
			//Not cached and no url specified then call error callback
			if(_validateExists(options.onErrorCallback))
			{
				options.onErrorCallback("No value found");
				unblock(options);
			}
		}

		function processError(error, options){
			$.each(dataQueue.pop(options.dataKey), function(){
				if(_validateExists(this.onErrorCallback))
				{
					this.onErrorCallback("Server request error. "+error);
					unblock(this);
				}
			});

			unblock(options);
		}

		function processDataSuccess(useCache, options, data)
		{
			if(!wait(useCache, options,data)){
				processData(useCache, options, data);
			}
		}

		function processData(useCache, options, data){
			if(useCache)
			{
				cache.put(options.dataKey, (options.onDataCallback)?options.onDataCallback(data, options):data, options);
				$.each(dataQueue.pop(options.dataKey), function(){
					_getData(this);
				});
			}
			else{
				if(options.timeToLive !== null && options.timeToLive < 0){
					if( _validatePopulated(options.cacheKey)){
						cache.remove(options.cacheKey);
					}
					if( _validatePopulated(options.requestKey)){
						cache.remove(options.requestKey);
					}
				}
				$.each(dataQueue.pop(options.dataKey), function(){
					if(_validateExists(this.onSuccessCallback))
					{
						this.onSuccessCallback((this.onDataCallback)?this.onDataCallback(data):data);
						unblock(this);
					}
				});
			}
		}

		function block(options){
			if(options.blockType !== null && !blockQueue.containsObject(options.blockType.getValue(),options)){
				blockQueue.push(options.blockType.getValue(), options);
				cache.clearType(options.blockType);
			}
		}

		function unblock(options){
			if(options.blockType !== null){
				blockQueue.popObject(options.blockType.getValue(), options);
				if(!blockQueue.contains(options.blockType.getValue()) && waitQueue.contains(options.blockType.getValue())){
					$.each(waitQueue.pop(options.blockType.getValue()), function(){
						processData(this.useCache, this.options, this.data);
					});
				}
			}
		}

		function wait(useCache, options, data){
			if(options.waitType !== null){

				if(blockQueue.contains(options.waitType.getValue())){

					waitQueue.push(options.waitType.getValue(), {options:options,data:data,useCache:useCache});
					return true;
				}
			}
			return false;
		}
	};

	var _validateExists = function(value){
		return ((typeof value) != "undefined") && value !== null;
	};

	var _validatePopulated = function(value){
		return _validateExists(value) && ((""+value).length)>0;
	};

	var _parseLocationReport = function(data, createReport, createDaySection) {
		var units = _parseReportUnits($(data).find('WxParamUnits'));
		var report;
		$(data).find('Location').each(function(){
			 var days = [];
			 $(this).find('Day').each(function(){
				 var times = [];
				 var dateString = _parseString(this,'date',true);
				 $(this).find('TimeStep').each(function(){
					times.push(_parseReportTime(this, units, dateString));
				 });
				 days.push(createDaySection({
					 day:this,
					 date:utility.ISO8601toDate(dateString),
					 times:times,
					 units:units
				}));
			 });

			report= createReport({
				report:this,
				location:_parseNumber(this,'id',true),
				days:days
			});
		});

		return report;
	};

	/**
	 * Creates a measurement object using the uom attribute and the text node of the provided
	 * data xml node.
	 * If element is epecified then a check is made for a child element (of data) of that name
	 * and that element will then be processed to create the measurement.
	 */
	 var _parseMeasurement = function(data, element, defaultUom){
		 var result = null,
             value,
             uom,
             root = null;
		 if((typeof element)!='undefined' && element)
		 {
			 var elem = $(data).find(element);
			 if((typeof elem)!='undefined' && elem && elem.length > 0)
			 {
				 root = elem[0];
			 }
		 }
		 else{
			 root = data;
		 }
		 value = _parseNumber(root);
		 uom = _parseString(root,'uom',true);
		 if((typeof uom) == 'undefined' || uom===null){
			 uom = defaultUom;
		 }

		 if(typeof value != 'undefined' && value!=null){
			 result = _doConversion({quantity:value,from:uom,to:defaultUom});
		 }
		 return result;
	 };

	 var _parseText = function(data,element,isAttribute){
		 var result = null;
		 if(isAttribute){
			 result = data.nodeType === 1 ?   // is this already a DOM node
                        data.getAttribute(element) :
                        $(data).attr(element);
         }else{
			 var root = null;
			 if( element ) {
                 var elem = $(data).find(element);
				 if(elem.length > 0) {
                     root = elem[0];
				 }
			 }else{
				 root = data;
			 }
			 if(root !== null){
				 result = root.nodeType === 1 ?   // is this already a DOM node
					 (root.childNodes[0]?root.childNodes[0].nodeValue:null) :
                     $(root).text();
			 }
		 }

		 return result !== null && result.length > 0 ? result : null;
	 };

	 var _parseNumber = function(data, element, isAttribute){
		 var text = _parseText(data, element, isAttribute);
		 var result = null;
		 if(text) {
             result = parseFloat(text);
		 }
		 return isNaN(result) ? null : result;
	 };

	 var _parseString = _parseText;

	 var _doConversion = function(args){
		 if(args.quantity !==null && args.from !==null && args.to !==null){
			 try{
				 return utility.convert(args);
			 }
			 catch(error){
				 return null;
			 }
		 }
		 else{
			return null;
		 }
	 };

	var _parseReportValues = function(values,units){
		var unitVals = units;
		if((typeof units)== 'undefined' || units == null ){
			unitVals = {};
		}
		if((typeof values) != 'undefined' && values)
		{
			return new reportValues(
			{
				weather:_parseNumber(values,'WeatherType'),
				windDirection:_parseString(values,'WindDirection'),
				windSpeed:_doConversion({quantity:_parseNumber(values,'WindSpeed'),from:units.windSpeed,to:_defaultSpeedUnit}),
				windGust:_doConversion({quantity:_parseNumber(values,'WindGust'),from:units.windGust,to:_defaultSpeedUnit}),
				visibility:_doConversion({quantity:_parseNumber(values,'Visibility'),from:units.visibility,to:_defaultDistanceUnit}),
				humidity:_doConversion({quantity:_parseNumber(values,'ScreenRelativeHumidity'),from:units.humidity,to:_defaultHumidityUnit}),
				temperature:_doConversion({quantity:_parseNumber(values,'Temperature'),from:units.temperature,to:_defaultTempUnit,round:false}),
				feelsLikeTemperature:_doConversion({quantity:_parseNumber(values,'FeelsLikeTemperature'),from:units.feelsLike,to:_defaultTempUnit,round:false}),
				uv:_parseNumber(values,'MaxUvIndex'),
				pressure:_doConversion({quantity:_parseNumber(values,'Pressure'),from:units.pressure,to:_defaultPressureUnit}),
				pressureTendency:pressureTendency.parse(_parseString(values,'PressureTendency'))
			});
		}

		return null;
	};

	var _parseReportTime = function(timeStep,units,dateString){
		var time = $(timeStep).attr("time");
		var datetime = null;
		if(_validatePopulated(time)&& _validatePopulated(dateString)){
			datetime = utility.ISO8601toDate(dateString+'T'+time+'z');
		}
		return new reportTime({
			time:datetime,
			reportValues:_parseReportValues(timeStep,units)});
	};

	var _parseReportUnits = function(data){
		var units = {};
		units.windGust = _parseString(data,'WindGust');
		units.pressure = _parseString(data,'Pressure');
		units.humidity = _parseString(data,'ScreenRelativeHumidity');
		units.temperature = _parseString(data,'Temperature');
		units.visibility = _parseString(data,'Visibility');
		units.windSpeed = _parseString(data,'WindSpeed');
		units.windDirection = _parseString(data,'WindDirection');
		units.feelsLike = _parseString(data,'FeelsLikeTemperature');

		return units;
	};

	function _requestId(){
		var _currentId = new Date().getTime();

		this.next = function()
		{
			return _currentId++;
		};
	}

	var requestId = new _requestId();


	//ASSIGN EXTERNAL FUNCTIONS

	// make enums public
	library.extremeType = extremeType;
	library.displayPeriod = displayPeriod;
	library.ppnType = ppnType;
	library.pressureTendency = pressureTendency;
	library.safetyType = safetyType;
	library.warningKind = warningKind;
	library.warningLevel = warningLevel;
	library.warningStatus = warningStatus;
	library.warningWeatherType = warningWeatherType;

	// make library namespaces / functions public
	library.extreme = extreme;
	library.forecast = forecast;
	library.forecastDay = forecastDay;
	library.forecasterCommentary = forecasterCommentary;
	library.groundConditionsImage = groundConditionsImage;
	library.location = location;
	library.locationDetail = locationDetail;
	library.log = _externalLog;
	library.mostSevereWarning = mostSevereWarning;
	library.mostSevereWarningType = mostSevereWarningType;
	library.mostSevereWarningDate = mostSevereWarningDate;
	library.mostSevereRegionalWarning = mostSevereRegionalWarning;
	library.mountain = mountain;
	library.mountainForecast = mountainForecast;
	library.mountainForecastDay = mountainForecastDay;
	library.mountainForecastHazard = mountainForecastHazard;
	library.mountainWeatherPeriod = mountainWeatherPeriod;
	library.nationalPark = nationalPark;
	library.nationalParkForecast = nationalParkForecast;
	library.nationalParkForecastSection = nationalParkForecastSection;
	library.observation = observation;
	library.observationDay = observationDay;
	library.point = point;
	library.rainGauge = rainGauge;
	library.rainGaugeReading = rainGaugeReading;
	library.region = region;
	library.regionalForecast = regionalForecast;
	library.regionalForecastSection = regionalForecastSection;
	library.reportTime = reportTime;
	library.reportValues = reportValues;
	library.timezone = timezone;
	library.utility = utility;
	library.warning = warning;
	library.warningRegion = warningRegion;
	library.weatherwatch = weatherwatch;
	library.weatherwatchURL = weatherwatchURL;
	library.zone = zone;

	library.useFQDN = function(useFQDN) {
		if (useFQDN == true) _urlRoot = _fqRoot; else _urlRoot = "";
		_setURLs();
	};

	library.setRootURL = function(rootURL) {
		if(_validatePopulated(rootURL)){
			_urlRoot = rootURL;
		}
		else{
			_urlRoot = "";
		}
		_setURLs();
	};

	library.setURLSuffix = function(suffix){
		if(_validatePopulated(suffix)){
			_urlSuffix = suffix;
		}
		else{
			_urlSufix = null;
		}
	};


	library.getTimezone = function(){
		return _timezone;
	};

	library.setTimezone = function(timezone){
		if(_validateExists(timezone) && _validateExists(timezone.getOffset())){
			_timezone = timezone;
		}
		else{
			_timezone = null;
		}
	};

	library.clearCache = function(){
		cache.clear();
	};

	function _buildURL(url){
		if(_urlSuffix !== null){
			return url+_urlSuffix;
		}
		else{
			return url;
		}
	}

	function _setURLs() {
		_daylightSavingURL = _urlRoot + "/public/data/InventCache/Item/DaylightSaving";
		_displayPeriodURL = _urlRoot + "/public/data/WCCache/Item/DisplayPeriod";
		_extremeURL = _urlRoot+"/public/data/PWSCache/UkExtremes/All";
		_forecastURL = _urlRoot+"/public/data/PWSCache/BestForecast/Forecast/";//+id
		_forecasterCommentaryURL = _urlRoot+"/public/data/PWSCache/ChiefForecastersView";
//		_locationURL = _urlRoot+"/public/data/PWSCache/Locations/MasterList";
		_locationURL = _urlRoot+"/public/data/PWSCache/PwsLocations/ForecastsList";
		_locationDetailURL = _urlRoot+"/public/data/PWSCache/PwsLocations/Details/"; //+id	
		_mountainForecastURL = _urlRoot+"/public/data/PWSCache/MountainForecast/Area/";//+id
		_nationalParkForecastRegionURL = _urlRoot + "/public/data/PWSCache/NationalParks/All";
		_observationURL = _urlRoot+"/public/data/PWSCache/BestForecast/Observation/";//+id
		_regionalForecastURL = _urlRoot+"/public/data/PWSCache/RegionalForecast/Area/";//+regionId
		_timezoneURL = _urlRoot+"/public/data/PWSCache/UtcOffset/";//+id
		_warningsURL = _urlRoot+"/public/data/PWSCache/Warnings/All";
	}

	return library;
})();
