
var newsFeed = startupWindow().newsFeed;

$(function() {
    log("Begin application");
    var dragged = false;
    initialiseWindow({close:onCloseWindow});

	$("#headersettings, #optionslink").click(function(){optionsWindow.show();});
	$("#headermoreless").click(changeSize);
    $('#headerabout').click(function(){aboutWindow.show();});

    $("#buttons").buttonset().find("input:radio").change(function(){
        log("buttons:change", this.value, this.checked);
        if( this.checked ){
            switch(this.value){
                case 'forecast':
                    showForecast();
                    break;
                case 'observations':
                    showObservations();
                    break;
                case 'warnings':
                    showWarnings();
                    break;
            }
        }
    });
    $('#forecast-timesteps a.back').click(function(e){
    	e.preventDefault();
    	showForecast();
    });
	$("#headernews").click(function(){newsWindow.show();});

    moListStyle = getELSParam("moListStyle") || "vertical";
    $(document.body).addClass(moListStyle);

    var nativeApp = air.NativeApplication.nativeApplication;
    nativeApp.addEventListener("optionsChanged",optionsChanged);
    
    if( newsFeed.unReadItemCount() > 0 ){
    	$('#headernews').addClass('highlight');
    }
	nativeApp.addEventListener('newNewsItemsAvailable',function(){
		$('#headernews').addClass('highlight');
	});
	nativeApp.addEventListener('noNewsItemsAvailable',function(){
		$('#headernews').removeClass('highlight');
	});
	
	initialize({showLoading:true});

	refreshTimers.startTimer(warningsRefreshTimeout, displayHeaderWarning);
	refreshTimers.startTimer(weatherRefreshTimeout, loadComponent);

    networkMonitor.up("mainWindow",displayHeaderWarning, true);
    networkMonitor.up("mainWindow",loadComponent,true);
    networkMonitor.up("mainWindow",function(){
        refreshTimers.restart()
    }, true);
    networkMonitor.up("mainWindow",function(){$('#networkWarning').stop().fadeOut();});

    networkMonitor.down("mainWindow",function(){
        $('#networkWarning').stop().fadeIn();
        refreshTimers.stop();
    });
    $('#networkWarning').tooltip(defaultTooltip("topleft"));

    if(!networkMonitor.available()){
        $('#content').append("<p class='networkWarning'>There is no network connection available.</p>");
        networkMonitor.up("mainWindow",function(){$('#content .networkWarning').remove();});
    }
});

function optionsChanged(){
    log("mainWindow reinitialize()");
    initialize({showLoading:true});
    if( $('#forecast-timesteps').is(':visible')){
        showForecast();
    }
    refreshTimers.restart();
}

function changeSize() {
	showForecast({syncButtons:true, animate:false});
    moListStyle = (moListStyle == "horizontal") ? "vertical": "horizontal";
	setELSParam("moListStyle", moListStyle);

    // reset the body class flags
    $(document.body).removeClass("horizontal vertical")
                    .addClass(moListStyle);
    
    recalculateSize();

    $('#buttons').buttonset('refresh');
}

function onCloseWindow() {
    networkMonitor.clear("mainWindow");
    refreshTimers.removeTimer(displayHeaderWarning);
	refreshTimers.removeTimer(loadComponent);
    air.NativeApplication.nativeApplication.removeEventListener("optionsChanged", optionsChanged);
	nativeWindow.close();
}

function calcHeight() {
	var height = 360;

	if (moListStyle == "horizontal") {
		height = 160;
	}
	
	return height;
}

function calcWidth() {
	var width = 280;
	//if (moMapDisplay == "true") {
		width += 190;
	//}
	if (moListStyle == "horizontal") {
		width = 160;
	}
	return width;
}

function daysToShow() {
	if (moListStyle == "horizontal") {
		return 3;
	} else {
		return moDays;
	}	
}
function recalculateSize(){
    setSize(calcWidth(),calcHeight());
}
function setSize(width, height) {
    nativeWindow.bounds = position(nativeWindow.bounds)
                            .merge({width:width,height:height})
                            .rectangle();
    
    displayForecastLocationText();
}
// Component javascript
// 
// Reads data and creates a simple tag representation of it, which is (mostly!) styled with CSS. 
//

var regions;
var w, h;
var requiredLanguage;
var moListStyle;
var moDays;
var moFSSI;
var moObId;
var moLocation = null;
var moMapDisplay;
var moMapsRequired;
var moTemperatureUnits;
var moSpeedUnits = "M";
var moShowWind;
var moShowUV;
var moShowFeelsLike;
var moAllowUserLocation;
var textWindUnits = "mph";
var textWindInUnits = "mph";
var labels = {};

var weathermapURL = "http://www.metoffice.gov.uk/weather/uk/uk_forecast_weather.html";
var maxFontSize = 18; // max font-size in px
var forecastLocationText = "";
var maxMinimalForecastLocationLength = 23;

var mapArea;
var displayPeriod;
var transparentFileSuffix = ($.browser.msie ? "gif" : "png");

function initialize(options){
    options = options || {};
	var userLanguage = getELSParam("moLanguage");
	if (userLanguage == null) {
		requiredLanguage = "en";
	}
	else {
		requiredLanguage = userLanguage;
	}
	setupLanguage(requiredLanguage, function(){
        processLanguage();
        getParameters();
        recalculateSize();
        loadComponent(options);
        displayHeaderWarning();
    });
}

function getParameters() {
	getDefaultParams();
	
	// set the map position
	if(moMapDisplay=="true"){
		$("#maps,#nonmap").addClass("side");
		$("#maps").show();
	}
	else{
		$("#maps,#nonmap").removeClass("side");
		$("#maps").hide();
	}
}
	
function loadComponent(options){
    options = options || {};
    log("loadComponent");

    if( networkMonitor.available()){
        // set up loading images & note
        if( options.showLoading ){
            $("#panel .location span,#obsContent .location").html(labels.general.loading);
            setForecastLocationText(labels.general.loading);
            $('#wxcontent').html("<img class='loader' src='/lib/images/ajaxLoaderLarge.gif' />");
            $('#obsTableDiv').html("<img class='loader' src='/lib/images/ajaxLoaderLarge.gif' />");
        }

        metoffice.region.getAll({
            success: function(regionList) {
                regions = {};
                for (var i = 0; i < regionList.length; i++) {
                    regions[regionList[i].getId()] = regionList[i].getName();
                }
                // Show day or night time?
                metoffice.utility.getDisplayPeriod({
                    success: function(result) {
                        log("getDisplayPeriod result", result.getValue());

                        displayPeriod = result;
                        loadWithDisplayPeriod();
                    },
                    error:showError
                });
            },
            error:showError
        });
    }
}

function loadWithDisplayPeriod() {
	if (moSpeedUnits){
        textWindUnits = labels.weather.wind[moSpeedUnits];
        textWindInUnits = labels.weather.windIn[moSpeedUnits];
	}

	// get the location name.
	// if not found, we show an error message.
	log("loadWithDisplayPeriod location ", currentLocation.get());
	processLocationData(currentLocation.get());

	// get the forecast data.
	// if not found, we show an error message.
	log("loadWithDisplayPeriod forecast");
	
	metoffice.forecast.getById({
		id: currentLocation.get().getId(),
		success: processWxData,
		error: showErrorForecast,
        timeToLive:ms.seconds(15)
	});
}

/**
* Process Obs Data
* Uses data from the obs location to build an html table populated with the following values: 
* Time, Weather Type, Wind Speed/Direction, Temperature and Visibility
*/
//TODO: Add unit conversions to table values
//TODO: Add units to table headers
//TODO: Fix time functions to display times correctly
//TODO: Add in language conversion functionality
//TODO: Add loading text and image whilst loading obs panel
//TODO: Add error handling so that message is shown when there's no data
function processObsData(location, obsLocation) {
	var obsDays = location.getObservationDays(), obsDate, obsTimes, obsValues, tbClass, obTimeDate,html, tips;
	log("processObsData", location, obsLocation);

    tips = $('#obsContent').data("tooltips");
    if( tips ){
        tips.tooltip('remove'); // clean these out first
    }

    $('#obsContent .location').html(internationalPlace(obsLocation.getName()));

	if (obsLocation.getId() != currentLocation.get().getId()) {
        $('#obsContent .nearest').html(labels.obs.nearest+" "+internationalPlace(currentLocation.get().getName()) + (moShowWind == "true" ? " &bull; " : "")).show();
    }else{
        $('#obsContent .nearest').hide();
    }
    if(moShowWind == "true"){
        $('#obsContent .wind-units').html(textWindInUnits).show();
    }else{
        $('#obsContent .wind-units').hide();
    }

	html = "<table class='weather' border='0' cellspacing='0' cellpadding='0'>";
	for (var d = (obsDays.length - 1); d >= 0; d--) {
		obsDate = internationalDate(obsDays[d].getDate()).slice(4,10);
		html += "<tr class='title'><td class='date'>" + obsDate + "</td>" +
                "<td>"+labels.weather.labels.weather+"</td>" +
                "<td>"+labels.weather.labels.temp+"</td>" +
                (moShowWind == "true" ? "<td>"+labels.weather.labels.wind+"</td>" : "") +
                "<td>"+labels.weather.labels.visibility+"</td></tr>";
		obsTimes = obsDays[d].getObservationTimes();
		log(obsDate);
		for (var t = (obsTimes.length - 1); t >= 0; t--) {
			tbClass = t%2==0 ? "even":tbClass = "odd";
			obTimeDate = obsTimes[t].getTime();
			obsValues = obsTimes[t].getReportValues();
			html += "<tr class='"+tbClass+"'>" +
					"<td>" + timeFormat(obTimeDate) + "</td>" +
					"<td>" + weather(obsValues.getWeather()).toMarkup() + "</td>" +
					"<td>" + temperature(obsValues.getTemperature(), moTemperatureUnits, undefined, false).toMarkup() + "</td>" +
					(moShowWind == "true" ? "<td>" + wind(obsValues.getWindSpeed(), obsValues.getWindDirection(), moSpeedUnits, textWindUnits).toMarkup() + "</td>" : "" ) +
					"<td>" + visibilityToWords(obsValues.getVisibility()) +"</td>" +
					"</tr>";
		}
	}
	html += "</table>";

	$('#obsTableDiv').html(html);
    tips = $('#obsContent *[title]').each(weatherSymbolToolTip);
    $('#obsTableDiv').data("tooltips", tips);
}

function processLanguage() {
	log("processLanguage");
    setLabelText();
    $("body").attr("dir", labels.language.direction)
        .attr("lang", labels.language.code);
    $("#componentCSS").remove();
    
    var headID = document.getElementsByTagName("head")[0];         
    var cssNode = document.createElement('link');
    cssNode.type = 'text/css';
    cssNode.rel = 'stylesheet';
    cssNode.href = 'component' + labels.language.direction + '.css';
    cssNode.media = 'screen';
	cssNode.id = "componentCSS";
    headID.appendChild(cssNode);
	
}

function setLabelText() {
	// options link
    $("#optionslink").attr("title", labels.link.alt.options);
    $("#optionslink").html(labels.link.text.options);

    // help link
    $("#helplink").attr("href", labels.link.url.help);
    $("#helplink").attr("title", labels.link.alt.help);
    $("#helplink").html(labels.link.text.help);

    // more details link
    $("#moredetaillink").attr("title", labels.link.alt.moredetails);
    $("#moredetaillink").html(labels.link.text.moredetails);

    // Copyright link
    $("#copyright a").attr("title", labels.link.alt.copyright);
    $("#copyright a").html(labels.link.text.copyright);
    $("#copyrightsymbol").html("&copy;");

    $("#logo a").attr("title", labels.link.alt.metoffice);
    $("#logo img").attr("alt", labels.link.alt.logo);
	
	$("#labelfordatatypeforecast").html(labels.reports.forecast);
	$("#labelfordatatypeobservations").html(labels.reports.observations);
	$("#labelfordatatypewarnings").html(labels.reports.warnings);
	
    $("#foreWeatherMessage").html(labels.weather.labels.weather);
    $("#foreTempMessage").html(labels.weather.labels.temp);
    $("#foreWindMessage").html(labels.weather.labels.wind);
    $("#foreFeelsLikeMessage").html(labels.weather.temperatureType.feelsLike);
    $("#foreUVMessage").html(labels.weather.labels.uv);
    $("#foreBackMessage").html(labels.forecast.back);
    
    // initialise the tooltips
    
    $('#header div.ui-icon[title]').tooltip(defaultTooltip("topright"));
    
}

function processMaps(xml) {
	var mapsArray = moMapsRequired.split(",");
	var multipleMaps = mapsArray.length > 1;
	var images = $(xml);
	var isImageOutput = false;
    
	$("#maps").html("").prepend("<div id=\"mapContainerGroup\">");
	for (var i in mapsArray) {
		var mapContainer = $("<div class=\"mapContainer\">");
		var layer = images.find("layer[type='" + mapsArray[i] + "']");
		var currentArea = ( layer.attr("type") == ("Pressure MSL") ? "wuk" : mapArea);
		var image = $(layer).find("image:first");
        var layerTitle = labels.layer[layer.attr("title")];
		if (image.length) {
			isImageOutput = true;
			var timestamp = new Date();
			timestamp.setISO8601(image.attr("timestamp"));
			var outputDate = timestamp.componentDateFormat(labels.calendar.month);
			mapContainer.append("<h2>" + layerTitle + "</h2>");
			mapContainer.append("<h3>" + outputDate + "</h3>");
			mapContainer.append("<div class=\"imageholder\">" + 
				"<img class=\"background\" src=\"/lib/images/maps/" + currentArea + ".png\">" + 
				"<img class=\"middle\" width=\"150\" height=\"150\" alt=\"" + layer.attr("title") + "\" src=\"" + component.formatImageSource(image.attr("src"), currentArea, transparentFileSuffix) + "\" />" + 
				(layer.attr("type") == ("Pressure MSL") ? "" : "<img class=\"foreground\" src=\"/lib/images/maps/" + currentArea + "Transparent." + transparentFileSuffix + "\">") + 
				"</div>" );
			mapContainer.appendTo("#mapContainerGroup");
		} else {
			mapContainer.append("<h2>" + layerTitle + "</h2>");
			mapContainer.append("<div class=\"imageholder\" style=\"background-color:#333333\">" + layer.attr("title") + labels.error.layernotretrievablepostamble + "</div>");
			mapContainer.appendTo("#mapContainerGroup");
		}
	}
	if (isImageOutput) {
		// wrap required instead of an inline href, because IE making overlays opaque with latter. 
		$(".imageholder img").attr("title", labels.link.alt.maps).wrap("<a target=\"_blank\" href=\"" + weathermapURL + "\" title=\"" + labels.link.alt.maps + "\"></a>");
		
		if (multipleMaps ) {
			$("#maps").prepend("<div class=\"previous nav\"><span><img src=\"/lib/images/left.jpg\"></span></div>");
			$("#maps").append("<div class=\"next nav\"><span><img src=\"/lib/images/right.jpg\"></span></div>");
			$(".next").click(nextMap);
			$(".previous").click(previousMap);
		} else {
			$(".nav").addClass("inactiveNav");
		}
		$(".mapContainer:first").addClass("current");
	}
}

function nextMap() {
	if (! $(".mapContainer.current").removeClass("current").next(".mapContainer").addClass("current").length) {
		$(".mapContainer:first").addClass("current");
	}
}

function previousMap() {
	if (! $(".mapContainer.current").removeClass("current").prev(".mapContainer").addClass("current").length) {
		$(".mapContainer:last").addClass("current");
	}
}


function processLocationData(location) {
	//$("#location").css("zoom", 1);	// IE hasLayout fix
	var region = location.getRegion();
  log("processLocationData location",location);
  
 log("processLocationData region",region);
    
	// get the location name from the data.xml xml.
	moLocation = location.getName();

//	setForecastLocationText(internationalPlace(location.getName()));
	setForecastLocationText(location.getName());
	//var regionId = region.getId();
	metoffice.locationDetail.getById({
		id: location.getId(),
		success: function(locDetailData) {
			mapArea = component.getAreaCode(location.getPoint().getLat(), location.getPoint().getLon(), locDetailData.getRegionId());
			$("#moredetaillink").attr("href", "http://www.metoffice.gov.uk/weather/uk/" + locDetailData.getRegionId() + "/" + locDetailData.getRegionId() + "_forecast_weather.html");  	// set up more details link

		},
		error: function(err) {
			log(err + ": Couldn't retrieve location details");
		},
		async: false
	});
	
	
	
	//$("#moredetaillink").attr("href", "http://www.metoffice.gov.uk/weather/uk/" + regionId + "/" + regionId + "_forecast_weather.html");  	// set up more details link

    currentLocation.nearestObservation({
		success:requestObsData, 
		error:showErrorObservations
	});
}

function requestObsData(result) {
    log("requestObsData",result);
	metoffice.observation.getById({
		id: result.location.getId(),
		success: function(data) {processObsData(data, result.location);},
		error: showErrorObservations,
        timeToLive:ms.seconds(15)
	});
}

function processWxData(data) {
	log("processWxData");

	//add the maps
	if (moMapsRequired !== "") {
		$.ajax({url: settings.rootDataUrl + "/public/data/WCCache/ItemList/Images/All",
			method: "GET",
			success: processMaps,
			cache: false
		});
	}

	// get the weather data out of the forecast xml
	var counter = 0 ;
    var displayDate = internationalDate(data.getForecastDays()[0].getDate());
	
	$("#moredetail").show();	// show the more detail link.
	$("#date").html(displayDate + ((moShowWind == "true") ? " &bull; " + textWindInUnits : "")).css("zoom", 1);	// show the date & wind units

    var previousTips = $('#wxcontent').data('tooltips');
    if( previousTips){
        previousTips.tooltip('remove'); // clean these out first
    }
	$("#wxcontent").html("<div class=\"days\"></div>");		// clear the panel ready for use
//	$("#content,#wxcontent").removeClass("horizontal vertical")
//				.addClass(moListStyle);	// the moListStyle class is added, 
	log("forecast data", data);
	for (var i = 0; i < data.getForecastDays().length; i++) {
		counter++;
		appendDay(data.getForecastDays()[i], counter);	// adds to the document another div with the day's data in
	}
											
	// set content to be in the middle (proving difficult in IE6 CSS)
	//$("#wxcontent").css("margin-left", ( $("#wxcontent").width() - $(".days").width() ) / 2 + "px");
	//below: work around to fix positioning of "more details" when there are less than 5 days available in side map view.
	if (moMapDisplay == "true" && counter < daysToShow()) {
		for (var c = counter;c<daysToShow();c++){
			var spaceFill = $("<div class=\"day\"><dl><dd></dd></dl></div>");
			spaceFill.appendTo("#wxcontent .days");
		}
	}

    $('.forecastIssuedAt').html("<span class='prefix'>Issued </span>" + data.getDataAt().componentDateFormat(labels.calendar.month));
    var tipped = $('#wxcontent *[title]').each(weatherSymbolToolTip);
    $('#wxcontent').data('tooltips', tipped);

}


function appendDay(day, counter) {
	// creates a div for this day and adds a dl to it.  The dl contains dl and dt for each type of reading (temp, wind, etc).  
	var dayElement = $("<div class='day'/>").addClass("day-" + counter);
	
	var dl = $("<dl></dl>");
	
	var thisDisplayPeriod = ((counter == 1) ? displayPeriod : metoffice.displayPeriod.DAY);	// for the first day, we show day or night 
																		// appropriately, but for the remainder we show day only.
	var data = day.valuesFor(thisDisplayPeriod);// === metoffice.displayPeriod.DAY ? day.getDayValues() : day.getNightValues();

    var dateTitle = internationalDate(day.getDate()) ;
	if (counter === 1) {
		dl.append("<dt>"+labels.weather.labels.date+"</dt><dd class='date displayPeriod" + thisDisplayPeriod + "'>" +
				"<span class='date' title='" + dateTitle + "'>" + internationalDisplayPeriod(thisDisplayPeriod) + "</span>" +
				"</dd>");
	} else {
		dl.append("<dt>"+labels.weather.labels.date+"</dt><dd class='date'>" +
                "<span class='date' title='" + dateTitle +"'>" + internationalDay(day.getDate()) + "</span></dd>");
	}
	dl.append("<dt>"+labels.weather.labels.weather+"</dt><dd class='weather'>" + weather(data.getWeather()).toMarkup() + "</dd>");
	dl.append("<dt>"+labels.weather.temperatureType.temperature+"</dt><dd class='temperature'>"+ temperature(data.getTemperature()).toMarkup() + "</dd>");

	if (moShowWind == "true") {
		dl.append("<dt>"+labels.weather.labels.weather+"</dt><dd class='wind'>" + wind(data.getWindSpeed(), data.getWindDirection(), moSpeedUnits, textWindUnits).toMarkup() + "</dd>");
	}
	if (moShowFeelsLike == "true") {
        dl.append("<dt>"+labels.weather.temperatureType.feelsLike+"</dt><dd class='feelsLike'>" + temperature(data.getFeelsLikeTemperature(), undefined, "Feels like").toMarkup() + "</dd>");
	}
	
	if (moShowUV == "true") {
		dl.append("<dt>"+labels.weather.labels.uv+"</dt><dd class='uv'>" + uv(data.getUv()).toMarkup() + "</dd>");
	} 
	
	dayElement.click(function(){
		if( moListStyle == 'vertical'){
			showForecastTimesteps(day,counter);
		}
	});
	dl.appendTo(dayElement);
	dayElement.appendTo("#wxcontent .days");
}

function showError(error) {
	// sets up the panel with an appropriate error mesage
	if (typeof(error) != "undefined") {
		air.trace("Error: " + error);
	}
    showErrorForecast(error, true);
    showErrorObservations(error, true);
    showForecast();
}

function showErrorObservations(error, suppress) {
	// sets up the panel with an appropriate error mesage
	if (typeof(error) != "undefined" && !suppress) {
		air.trace("Error: " + error);
	}
    $('#obsContent .location').html(labels.error.wxnotfound);
    $('#obsTableDiv').html("");
}

function showErrorForecast(error, suppress) {
	// sets up the panel with an appropriate error mesage
	if (typeof(error) != "undefined" && !suppress) {
		air.trace("Error: " + error);
	}
	$("#moredetail").show();
	setForecastLocationText(labels.error.wxnotfound);
    var textLocation = (moLocation === null || (typeof(moLocation) == "undefined") ? labels.general.location : internationalPlace(moLocation));

	$("#wxcontent").html("<div class=\"wxnotfound\">" + labels.error.wxnotfoundpreamble + " " + textLocation + " " + labels.error.wxnotfoundpostamble + "</div>");
//    showForecast();
}

function setForecastLocationText(text){
	forecastLocationText = text;
	displayForecastLocationText();
}

function displayForecastLocationText(){
	var $locations = $("#panel .location span");
	var text = forecastLocationText;
	if(moListStyle == "horizontal"){	
		if(	text.length > maxMinimalForecastLocationLength){
			text = text.substring(0,maxMinimalForecastLocationLength)+".";
		}
	}
	
	$locations.html(text);
	resizeText();
}

function resizeText() {
	var $locations = $("#panel .location span");
	
	$locations.css("font-size", maxFontSize + "px");
	var currentWidth = $locations.width();
	var desiredWidth = 300;
	if(moListStyle == "horizontal"){	
		desiredWidth = 150;
	}
	var newFontSize = (desiredWidth / currentWidth) * maxFontSize;
	$locations.css("font-size", Math.min(newFontSize,maxFontSize)+ "px");
}

function showPanelContent(name, animate){
    if( typeof animate ==='undefined'){
        animate = true;
    }
    var $panel = $('#' + name).stop(true, true);
    var $siblings = $panel.siblings();

    $siblings.stop(true, true);

    $siblings.trigger('hiding');
    $panel.trigger('showing');
    if(animate){
        $siblings.fadeOut('fast');
        $panel.delay(500).fadeIn('fast');
    }else{
        $siblings.hide();
        $panel.show();
    }
}

function showForecast(options){
    options = options || {animate:true};
    if( options.syncButtons){
        $('#datatypeforecast').attr("checked",true).change();
        $('#buttons').buttonset('refresh');
    }
	showPanelContent("content", options.animate);
}

function showObservations(){
	showPanelContent("obsContent");
}

function showWarnings(options) {
	options = options || {};
    if( moListStyle !== "vertical"){
        changeSize();
    }
	if( options.syncButtons ){
        $('#datatypewarnings').attr("checked",true).change();
        $('#buttons').buttonset('refresh');
	}
	showPanelContent("warnings");
	$("#warnings").html("<img src=\"lib/images/ajaxLoaderLarge.gif\">");
	getWarnings();
}

function getWarnings() {
	metoffice.warning.getAllWarnings({
        timeToLive: ms.seconds(15),
		success: processWarnings,
		error: processWarningsError
	});
}

function processWarningsError() {
	$("#warnings").html("<p>"+labels.error.warningsDownloadFailed+"</p>");
}

function processWarnings(warnings) {
	if (warnings.length) {
		outputWarnings(warnings);
	} else {
		outputNoWarnings();
	}
    // make the header & icon are in sync
    displayHeaderWarning();
    startupWindow().updateWarnings();
}

function outputWarnings(warnings) {
	$("#warnings").html("<p>"+labels.warnings.warningsInForce+"</p>");
	$("#warnings").append("<table cellspacing=\"0\" cellpadding=\"0\"><thead><tr><th>"+labels.warnings.region+"</th><th>"+labels.warnings.hazards+"</th><th>"+labels.warnings.validity+"</th></tr></thead><tbody id=\"warningsrows\"></tbody></table>");
    var warningRows = [];
    // build a list of the requied data
    info("Warnings:", warnings);
	warnings.forEach(function(warning) {
        warning.getRegions().forEach(function(region) {
            warningRows.push({
                level:warning.getMaxLevelByRegion(region.getId()),
                kind:warning.getKind(),
                region:region,
                weatherType:warning.getWeatherType(),
                validFrom:warning.getValidFrom(),
                validTo:warning.getValidTo()
            });
        });
    });
    // sort my level (most serve first) then by validFrom (earlest first)
    warningRows.sort(function(a,b){
        var r = b.level.getValue() - a.level.getValue();
        if( r === 0){
            r = a.validFrom.getTime() - b.validFrom.getTime();
        }
        return r;
    });

    // output the data as a table row for each item
    warningRows.forEach(function(data){
       $("<tr class='" + data.level.toString().toLowerCase() + "'>" +
               "<td>" + internationalPlace(regions[data.region.getId()]) + "</td>" +
               "<td nowrap>" + capitalise(data.level) + " " + capitalise(data.kind) + " of " + capitalise(data.weatherType) + "</td>" +
               "<td nowrap>" + metoffice.utility.dateFormat(data.validFrom) + " - " + metoffice.utility.dateFormat(data.validTo) + "</td>" +
           "</tr>")
           .click(function(){
               openExternalURL("http://www.metoffice.gov.uk/weather/uk/" + data.region.getId() +"/" + data.region.getId() + "_forecast_warnings.html");
           })
           .appendTo("#warningsrows");
   });
}

function outputNoWarnings() {
	$("#warnings").html("<p>"+labels.warnings.noWarningsInForce+"</p>");
}

function displayHeaderWarning() {
    log("displayHeaderWarning");
    if( networkMonitor.available()){
        metoffice.warning.getMaxWarningLevel({
            timeToLive: ms.seconds(15),
            success: function(result) {
                info("getMaxWarningLevel - " + result);
                // for what ever reason we have to supply the label again to
                // the button, otherwise it gets overwritten on the third tuesday of the month
                if (result == metoffice.warningLevel.GREEN) {
                    $("#datatypewarnings").button( "option", {
                                                "icons":{primary:''},
                                                "label": labels.reports.warnings
                                            });
                } else {
    //				alert("getMaxWarningLevel = "+ labels.reports.warnings);
                    $("#datatypewarnings").addClass("ui-state-error ui-widget")
                                            .button("option", {
                                                "icons":{primary:'ui-icon-alert'},
                                                "label": labels.reports.warnings
                                            });
                }
            },
            error: function() {
                $("#warningtext").text(labels.warnings.warningsNotAvailable);
            }
        });
        function showHeaderWarnings(){
            $("#warning:hidden").find("img[title]")
                                    .tooltip(defaultTooltip("topright"))
                                .end()
                                .click(function(){
                                    showWarnings({syncButtons:true});
                                })
                                .fadeIn("slow");
        }
        metoffice.warning.getMaxWarningLevelByAllTypes({
            timeToLive: ms.seconds(15),
            success: function(results) {
                var resultsByType = {};
                // build a map of levels
                results.forEach(function(obj){
                    if( obj.level !== null ){
                        resultsByType[obj.type.toString().toLowerCase()] = obj;
                    }
                });

                // set the image icons for the all levels
                $("#warning td img").each(function(){
                    var $img = $(this),
                        type = $img.attr("data-type"),
                        result = resultsByType[type],
                        level = result ? result.level : metoffice.warningLevel.GREEN,
                        kind = result ? result.kind : null,
                        tip ,
                        tooltip = $img.data("tooltip");
                    if( level !== metoffice.warningLevel.GREEN ){
                        tip = capitalise(level) + " " + capitalise(kind) + " of " + capitalise(type);
                    }else{
                        tip =  labels.warnings.typeLabels[type]['Green'];
                    }
                    $img.attr("src", "lib/images/warnings/" + type + "-" + level.toString().toLowerCase() + ".gif")
                        .attr("title", tip);

                    // update the tool tip if present
                    if( tooltip && tooltip.getTip()){
                        tooltip.getTip().html(tip);
                    }
                });

                showHeaderWarnings();
            },
            error: function() {
                $("#warning td img").each(function(){
                    var $img = $(this),
                        type = $img.attr("data-type"),
                        tooltip = $img.data("tooltip");

                    $img.attr("src", "lib/images/warnings/" + type + "-off.gif")
                        .attr("title", labels.warnings.warningsNotAvailable);

                    // update the tool tip if present
                    if( tooltip && tooltip.getTip()){
                        tooltip.getTip().html(labels.warnings.warningsNotAvailable);
                    }
                });
                showHeaderWarnings();
            }
        });
    }
}


function showForecastTimesteps(day){
	var $stepsTable, times, now = Date.now(), tips;
	log("showForecastTimesteps");
	$('#forecast-timesteps table thead .date-title').html(internationalDate(day.getDate()).slice(4,10));
	$stepsTable = $('#forecast-timesteps table tbody');

    tips = $stepsTable.data("tooltips");
    if( tips ){
        tips.tooltip('remove');
    }
	$stepsTable.children().remove();

    $('#foreWindMessage')[moShowWind == "true" ? 'show' : 'hide']();
    $('#foreFeelsLikeMessage')[moShowFeelsLike == "true" ? 'show' : 'hide']();
    $('#foreUVMessage')[moShowUV == "true" ? 'show' : 'hide']();

	showPanelContent("forecast-timesteps");
	
	$('#forecast-timesteps .tagline').html(internationalDate(day.getDate()) + (moShowWind == "true" ? " &bull; " + textWindInUnits : ""));
	
	times = day.getForecastTimes();
	
	$.each(times, function(index, value){
        if( value.getTime().getTime() > now ){
            var $row = $("<tr/>", {'class':index % 2 == 0 ? 'even' : 'odd'});
            var td = function(name){ return $("<td class='" + name + "'></td>").appendTo($row);};
            var report = value.getReportValues();
            td('time').html(timeFormat(value.getTime()));
            td('weather').html(weather(report.getWeather()).toMarkup());

            td('temperature').html(temperature(report.getTemperature(), moTemperatureUnits).toMarkup());
            if (moShowWind == "true") {
                td('wind').html(wind(report.getWindSpeed(), report.getWindDirection(), moSpeedUnits, textWindUnits).toMarkup());
            }
            if (moShowFeelsLike == "true") {
                td('feels-like').html(temperature(report.getFeelsLikeTemperature(), moTemperatureUnits, "Feels like").toMarkup());
            }
            if (moShowUV == "true") {
                td('uv').html(uv(report.getUv()).toMarkup());
            }
            $row.appendTo($stepsTable);
        }
	});
    tips = $stepsTable.find('*[title]').each(weatherSymbolToolTip);
    $stepsTable.data("tooltips", tips);
}


function weatherSymbolToolTip(element){
    var $e = $(this),
        offset = [0,0],
        tipClass = "topleft2-tooltip";
    if( $e.hasClass("wind")){
        offset = [-10,-10];
    }else if( $e.hasClass("date")){
        offset = [0,-15];
    }else if ($e.hasClass("temperature") || $e.hasClass("weather")){
        offset = [5,-10];
    }else if ($e.hasClass("uv")){
        tipClass = tipClass + " wide";
        offset = [-5,-15];
    }
    $e.tooltip(defaultTooltip("",{
        onBeforeShow:function(){ return moListStyle === 'vertical'; },
        position:"bottom right",
        offset:offset,
        effect:"toggle",
        tipClass:tipClass
    })).dynamic();

}